/* hashmap.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from hashmap.vala, do not modify */

/* hashmap.vala
 *
 * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
 * Copyright (C) 1997-2000  GLib Team and others
 * Copyright (C) 2007-2009  Jürg Billeter
 * Copyright (C) 2009-2014  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_MAP (gee_map_get_type ())
#define GEE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP, GeeMap))
#define GEE_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP))
#define GEE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP, GeeMapIface))

typedef struct _GeeMap GeeMap;
typedef struct _GeeMapIface GeeMapIface;

#define GEE_TYPE_MAP_ITERATOR (gee_map_iterator_get_type ())
#define GEE_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIterator))
#define GEE_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP_ITERATOR))
#define GEE_MAP_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIteratorIface))

typedef struct _GeeMapIterator GeeMapIterator;
typedef struct _GeeMapIteratorIface GeeMapIteratorIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_SET (gee_set_get_type ())
#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))

typedef struct _GeeSet GeeSet;
typedef struct _GeeSetIface GeeSetIface;

#define GEE_MAP_TYPE_ENTRY (gee_map_entry_get_type ())
#define GEE_MAP_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntry))
#define GEE_MAP_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))
#define GEE_MAP_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))

typedef struct _GeeMapEntry GeeMapEntry;
typedef struct _GeeMapEntryClass GeeMapEntryClass;

#define GEE_TYPE_ABSTRACT_MAP (gee_abstract_map_get_type ())
#define GEE_ABSTRACT_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMap))
#define GEE_ABSTRACT_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMapClass))
#define GEE_IS_ABSTRACT_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_MAP))
#define GEE_IS_ABSTRACT_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_MAP))
#define GEE_ABSTRACT_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMapClass))

typedef struct _GeeAbstractMap GeeAbstractMap;
typedef struct _GeeAbstractMapClass GeeAbstractMapClass;
typedef struct _GeeAbstractMapPrivate GeeAbstractMapPrivate;

#define GEE_TYPE_HASH_MAP (gee_hash_map_get_type ())
#define GEE_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_HASH_MAP, GeeHashMap))
#define GEE_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_HASH_MAP, GeeHashMapClass))
#define GEE_IS_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_HASH_MAP))
#define GEE_IS_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_HASH_MAP))
#define GEE_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_HASH_MAP, GeeHashMapClass))

typedef struct _GeeHashMap GeeHashMap;
typedef struct _GeeHashMapClass GeeHashMapClass;
typedef struct _GeeHashMapPrivate GeeHashMapPrivate;
typedef struct _GeeHashMapNode GeeHashMapNode;

#define GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE (gee_functions_hash_data_func_closure_get_type ())
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosure))
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_HASH_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_HASH_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_HASH_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_HASH_DATA_FUNC_CLOSURE, GeeFunctionsHashDataFuncClosureClass))

typedef struct _GeeFunctionsHashDataFuncClosure GeeFunctionsHashDataFuncClosure;
typedef struct _GeeFunctionsHashDataFuncClosureClass GeeFunctionsHashDataFuncClosureClass;

#define GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE (gee_functions_equal_data_func_closure_get_type ())
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosure))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))

typedef struct _GeeFunctionsEqualDataFuncClosure GeeFunctionsEqualDataFuncClosure;
typedef struct _GeeFunctionsEqualDataFuncClosureClass GeeFunctionsEqualDataFuncClosureClass;
#define _gee_functions_hash_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_hash_data_func_closure_unref (var), NULL)))
#define _gee_functions_equal_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_equal_data_func_closure_unref (var), NULL)))
#define _gee_hash_map_node_free0(var) ((var == NULL) ? NULL : (var = (gee_hash_map_node_free (var), NULL)))

#define GEE_HASH_MAP_TYPE_NODE_ITERATOR (gee_hash_map_node_iterator_get_type ())
#define GEE_HASH_MAP_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator))
#define GEE_HASH_MAP_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIteratorClass))
#define GEE_HASH_MAP_IS_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR))
#define GEE_HASH_MAP_IS_NODE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_NODE_ITERATOR))
#define GEE_HASH_MAP_NODE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIteratorClass))

typedef struct _GeeHashMapNodeIterator GeeHashMapNodeIterator;
typedef struct _GeeHashMapNodeIteratorClass GeeHashMapNodeIteratorClass;

#define GEE_HASH_MAP_TYPE_MAP_ITERATOR (gee_hash_map_map_iterator_get_type ())
#define GEE_HASH_MAP_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator))
#define GEE_HASH_MAP_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIteratorClass))
#define GEE_HASH_MAP_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR))
#define GEE_HASH_MAP_IS_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_MAP_ITERATOR))
#define GEE_HASH_MAP_MAP_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIteratorClass))

typedef struct _GeeHashMapMapIterator GeeHashMapMapIterator;
typedef struct _GeeHashMapMapIteratorClass GeeHashMapMapIteratorClass;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;

#define GEE_TYPE_ABSTRACT_SET (gee_abstract_set_get_type ())
#define GEE_ABSTRACT_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_SET, GeeAbstractSet))
#define GEE_ABSTRACT_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_SET, GeeAbstractSetClass))
#define GEE_IS_ABSTRACT_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_SET))
#define GEE_IS_ABSTRACT_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_SET))
#define GEE_ABSTRACT_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_SET, GeeAbstractSetClass))

typedef struct _GeeAbstractSet GeeAbstractSet;
typedef struct _GeeAbstractSetClass GeeAbstractSetClass;

#define GEE_HASH_MAP_TYPE_KEY_SET (gee_hash_map_key_set_get_type ())
#define GEE_HASH_MAP_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet))
#define GEE_HASH_MAP_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))
#define GEE_HASH_MAP_IS_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_IS_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_KEY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))

typedef struct _GeeHashMapKeySet GeeHashMapKeySet;
typedef struct _GeeHashMapKeySetClass GeeHashMapKeySetClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_HASH_MAP_TYPE_VALUE_COLLECTION (gee_hash_map_value_collection_get_type ())
#define GEE_HASH_MAP_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection))
#define GEE_HASH_MAP_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))

typedef struct _GeeHashMapValueCollection GeeHashMapValueCollection;
typedef struct _GeeHashMapValueCollectionClass GeeHashMapValueCollectionClass;

#define GEE_HASH_MAP_TYPE_ENTRY_SET (gee_hash_map_entry_set_get_type ())
#define GEE_HASH_MAP_ENTRY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet))
#define GEE_HASH_MAP_ENTRY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySetClass))
#define GEE_HASH_MAP_IS_ENTRY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET))
#define GEE_HASH_MAP_IS_ENTRY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY_SET))
#define GEE_HASH_MAP_ENTRY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySetClass))

typedef struct _GeeHashMapEntrySet GeeHashMapEntrySet;
typedef struct _GeeHashMapEntrySetClass GeeHashMapEntrySetClass;
typedef struct _GeeFunctionsHashDataFuncClosurePrivate GeeFunctionsHashDataFuncClosurePrivate;
typedef struct _GeeFunctionsEqualDataFuncClosurePrivate GeeFunctionsEqualDataFuncClosurePrivate;
typedef struct _GeeMapEntryPrivate GeeMapEntryPrivate;

#define GEE_HASH_MAP_TYPE_ENTRY (gee_hash_map_entry_get_type ())
#define GEE_HASH_MAP_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry))
#define GEE_HASH_MAP_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntryClass))
#define GEE_HASH_MAP_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY))
#define GEE_HASH_MAP_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY))
#define GEE_HASH_MAP_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntryClass))

typedef struct _GeeHashMapEntry GeeHashMapEntry;
typedef struct _GeeHashMapEntryClass GeeHashMapEntryClass;
typedef struct _GeeHashMapEntryPrivate GeeHashMapEntryPrivate;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;
typedef struct _GeeAbstractSetPrivate GeeAbstractSetPrivate;
typedef struct _GeeHashMapKeySetPrivate GeeHashMapKeySetPrivate;

#define GEE_HASH_MAP_TYPE_KEY_ITERATOR (gee_hash_map_key_iterator_get_type ())
#define GEE_HASH_MAP_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator))
#define GEE_HASH_MAP_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))
#define GEE_HASH_MAP_IS_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_IS_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_KEY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))

typedef struct _GeeHashMapKeyIterator GeeHashMapKeyIterator;
typedef struct _GeeHashMapKeyIteratorClass GeeHashMapKeyIteratorClass;
typedef struct _GeeHashMapValueCollectionPrivate GeeHashMapValueCollectionPrivate;

#define GEE_HASH_MAP_TYPE_VALUE_ITERATOR (gee_hash_map_value_iterator_get_type ())
#define GEE_HASH_MAP_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator))
#define GEE_HASH_MAP_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_VALUE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))

typedef struct _GeeHashMapValueIterator GeeHashMapValueIterator;
typedef struct _GeeHashMapValueIteratorClass GeeHashMapValueIteratorClass;
typedef struct _GeeHashMapEntrySetPrivate GeeHashMapEntrySetPrivate;

#define GEE_HASH_MAP_TYPE_ENTRY_ITERATOR (gee_hash_map_entry_iterator_get_type ())
#define GEE_HASH_MAP_ENTRY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator))
#define GEE_HASH_MAP_ENTRY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIteratorClass))
#define GEE_HASH_MAP_IS_ENTRY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR))
#define GEE_HASH_MAP_IS_ENTRY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR))
#define GEE_HASH_MAP_ENTRY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIteratorClass))

typedef struct _GeeHashMapEntryIterator GeeHashMapEntryIterator;
typedef struct _GeeHashMapEntryIteratorClass GeeHashMapEntryIteratorClass;
typedef struct _GeeHashMapNodeIteratorPrivate GeeHashMapNodeIteratorPrivate;
typedef struct _GeeHashMapKeyIteratorPrivate GeeHashMapKeyIteratorPrivate;
typedef struct _GeeHashMapMapIteratorPrivate GeeHashMapMapIteratorPrivate;
typedef struct _GeeHashMapValueIteratorPrivate GeeHashMapValueIteratorPrivate;
typedef struct _GeeHashMapEntryIteratorPrivate GeeHashMapEntryIteratorPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

typedef gboolean (*GeeForallFunc) (gpointer g, void* user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy** lazy, void* user_data);
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, void* user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, void* user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, void* user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, void* user_data);
struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, void* f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, int* result_length1);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

typedef gpointer (*GeeFoldMapFunc) (gconstpointer k, gconstpointer v, gpointer a, void* user_data);
typedef gboolean (*GeeForallMapFunc) (gconstpointer k, gconstpointer v, void* user_data);
struct _GeeMapIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMapIterator* self);
	GType (*get_v_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMapIterator* self);
	gboolean (*next) (GeeMapIterator* self);
	gboolean (*has_next) (GeeMapIterator* self);
	gpointer (*get_key) (GeeMapIterator* self);
	gpointer (*get_value) (GeeMapIterator* self);
	void (*set_value) (GeeMapIterator* self, gconstpointer value);
	void (*unset) (GeeMapIterator* self);
	gpointer (*fold) (GeeMapIterator* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldMapFunc f, void* f_target, gpointer seed);
	gboolean (*foreach) (GeeMapIterator* self, GeeForallMapFunc f, void* f_target);
	gboolean (*get_valid) (GeeMapIterator* self);
	gboolean (*get_mutable) (GeeMapIterator* self);
	gboolean (*get_read_only) (GeeMapIterator* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeSetIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeSet* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeSet* self);
	GDestroyNotify (*get_g_destroy_func) (GeeSet* self);
	GeeSet* (*get_read_only_view) (GeeSet* self);
};

struct _GeeMapIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMap* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMap* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMap* self);
	GType (*get_v_type) (GeeMap* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMap* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMap* self);
	gboolean (*has_key) (GeeMap* self, gconstpointer key);
	gboolean (*has) (GeeMap* self, gconstpointer key, gconstpointer value);
	gpointer (*get) (GeeMap* self, gconstpointer key);
	void (*set) (GeeMap* self, gconstpointer key, gconstpointer value);
	gboolean (*unset) (GeeMap* self, gconstpointer key, gpointer* value);
	void (*clear) (GeeMap* self);
	GeeMapIterator* (*map_iterator) (GeeMap* self);
	void (*set_all) (GeeMap* self, GeeMap* map);
	gboolean (*unset_all) (GeeMap* self, GeeMap* map);
	gboolean (*has_all) (GeeMap* self, GeeMap* map);
	gint (*get_size) (GeeMap* self);
	gboolean (*get_is_empty) (GeeMap* self);
	gboolean (*get_read_only) (GeeMap* self);
	GeeSet* (*get_keys) (GeeMap* self);
	GeeCollection* (*get_values) (GeeMap* self);
	GeeSet* (*get_entries) (GeeMap* self);
	GeeMap* (*get_read_only_view) (GeeMap* self);
};

struct _GeeAbstractMap {
	GObject parent_instance;
	GeeAbstractMapPrivate * priv;
};

struct _GeeAbstractMapClass {
	GObjectClass parent_class;
	gboolean (*has_key) (GeeAbstractMap* self, gconstpointer key);
	gboolean (*has) (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
	gpointer (*get) (GeeAbstractMap* self, gconstpointer key);
	void (*set) (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
	gboolean (*unset) (GeeAbstractMap* self, gconstpointer key, gpointer* value);
	GeeMapIterator* (*map_iterator) (GeeAbstractMap* self);
	void (*clear) (GeeAbstractMap* self);
	gboolean (*foreach) (GeeAbstractMap* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeAbstractMap* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	void (*reserved0) (GeeAbstractMap* self);
	void (*reserved1) (GeeAbstractMap* self);
	void (*reserved2) (GeeAbstractMap* self);
	void (*reserved3) (GeeAbstractMap* self);
	void (*reserved4) (GeeAbstractMap* self);
	void (*reserved5) (GeeAbstractMap* self);
	void (*reserved6) (GeeAbstractMap* self);
	void (*reserved7) (GeeAbstractMap* self);
	void (*reserved8) (GeeAbstractMap* self);
	void (*reserved9) (GeeAbstractMap* self);
	gint (*get_size) (GeeAbstractMap* self);
	gboolean (*get_read_only) (GeeAbstractMap* self);
	GeeSet* (*get_keys) (GeeAbstractMap* self);
	GeeCollection* (*get_values) (GeeAbstractMap* self);
	GeeSet* (*get_entries) (GeeAbstractMap* self);
	GeeMap* (*get_read_only_view) (GeeAbstractMap* self);
};

struct _GeeHashMap {
	GeeAbstractMap parent_instance;
	GeeHashMapPrivate * priv;
};

struct _GeeHashMapClass {
	GeeAbstractMapClass parent_class;
};

struct _GeeHashMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	gint _array_size;
	gint _nnodes;
	GeeHashMapNode** _nodes;
	gint _nodes_length1;
	gint __nodes_size_;
	GeeFunctionsHashDataFuncClosure* _key_hash_func;
	GeeFunctionsEqualDataFuncClosure* _key_equal_func;
	GeeFunctionsEqualDataFuncClosure* _value_equal_func;
	GeeSet* _keys;
	GeeCollection* _values;
	GeeSet* _entries;
	gint _stamp;
};

typedef guint (*GeeHashDataFunc) (gconstpointer v, void* user_data);
typedef gboolean (*GeeEqualDataFunc) (gconstpointer a, gconstpointer b, void* user_data);
struct _GeeHashMapNode {
	gpointer key;
	gpointer value;
	GeeHashMapNode* next;
	guint key_hash;
	GeeMapEntry* entry;
};

struct _GeeFunctionsHashDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsHashDataFuncClosurePrivate * priv;
	GeeHashDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsHashDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsHashDataFuncClosure *self);
};

struct _GeeFunctionsEqualDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsEqualDataFuncClosurePrivate * priv;
	GeeEqualDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsEqualDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsEqualDataFuncClosure *self);
};

struct _GeeMapEntry {
	GObject parent_instance;
	GeeMapEntryPrivate * priv;
};

struct _GeeMapEntryClass {
	GObjectClass parent_class;
	gconstpointer (*get_key) (GeeMapEntry* self);
	gconstpointer (*get_value) (GeeMapEntry* self);
	void (*set_value) (GeeMapEntry* self, gconstpointer value);
	gboolean (*get_read_only) (GeeMapEntry* self);
};

struct _GeeHashMapEntry {
	GeeMapEntry parent_instance;
	GeeHashMapEntryPrivate * priv;
};

struct _GeeHashMapEntryClass {
	GeeMapEntryClass parent_class;
};

struct _GeeHashMapEntryPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMapNode* _node;
};

struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gboolean (*foreach) (GeeAbstractCollection* self, GeeForallFunc f, void* f_target);
	void (*reserved0) (GeeAbstractCollection* self);
	void (*reserved1) (GeeAbstractCollection* self);
	void (*reserved2) (GeeAbstractCollection* self);
	void (*reserved3) (GeeAbstractCollection* self);
	void (*reserved4) (GeeAbstractCollection* self);
	void (*reserved5) (GeeAbstractCollection* self);
	void (*reserved6) (GeeAbstractCollection* self);
	void (*reserved7) (GeeAbstractCollection* self);
	void (*reserved8) (GeeAbstractCollection* self);
	void (*reserved9) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_read_only) (GeeAbstractCollection* self);
	GeeCollection* (*get_read_only_view) (GeeAbstractCollection* self);
};

struct _GeeAbstractSet {
	GeeAbstractCollection parent_instance;
	GeeAbstractSetPrivate * priv;
};

struct _GeeAbstractSetClass {
	GeeAbstractCollectionClass parent_class;
	void (*reserved0) (GeeAbstractSet* self);
	void (*reserved1) (GeeAbstractSet* self);
	void (*reserved2) (GeeAbstractSet* self);
	void (*reserved3) (GeeAbstractSet* self);
	void (*reserved4) (GeeAbstractSet* self);
	void (*reserved5) (GeeAbstractSet* self);
	void (*reserved6) (GeeAbstractSet* self);
	void (*reserved7) (GeeAbstractSet* self);
	void (*reserved8) (GeeAbstractSet* self);
	void (*reserved9) (GeeAbstractSet* self);
	GeeSet* (*get_read_only_view) (GeeAbstractSet* self);
};

struct _GeeHashMapKeySet {
	GeeAbstractSet parent_instance;
	GeeHashMapKeySetPrivate * priv;
};

struct _GeeHashMapKeySetClass {
	GeeAbstractSetClass parent_class;
};

struct _GeeHashMapKeySetPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapValueCollection {
	GeeAbstractCollection parent_instance;
	GeeHashMapValueCollectionPrivate * priv;
};

struct _GeeHashMapValueCollectionClass {
	GeeAbstractCollectionClass parent_class;
};

struct _GeeHashMapValueCollectionPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapEntrySet {
	GeeAbstractSet parent_instance;
	GeeHashMapEntrySetPrivate * priv;
};

struct _GeeHashMapEntrySetClass {
	GeeAbstractSetClass parent_class;
};

struct _GeeHashMapEntrySetPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeHashMap* _map;
};

struct _GeeHashMapNodeIterator {
	GObject parent_instance;
	GeeHashMapNodeIteratorPrivate * priv;
	GeeHashMap* _map;
	gint _index;
	GeeHashMapNode* _node;
	GeeHashMapNode* _next;
	gint _stamp;
};

struct _GeeHashMapNodeIteratorClass {
	GObjectClass parent_class;
	gboolean (*get_read_only) (GeeHashMapNodeIterator* self);
};

struct _GeeHashMapNodeIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapKeyIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapKeyIteratorPrivate * priv;
};

struct _GeeHashMapKeyIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapKeyIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapMapIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapMapIteratorPrivate * priv;
};

struct _GeeHashMapMapIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapMapIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapValueIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapValueIteratorPrivate * priv;
};

struct _GeeHashMapValueIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapValueIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeHashMapEntryIterator {
	GeeHashMapNodeIterator parent_instance;
	GeeHashMapEntryIteratorPrivate * priv;
};

struct _GeeHashMapEntryIteratorClass {
	GeeHashMapNodeIteratorClass parent_class;
};

struct _GeeHashMapEntryIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};


static gpointer gee_hash_map_parent_class = NULL;
static gpointer gee_hash_map_entry_parent_class = NULL;
static gpointer gee_hash_map_key_set_parent_class = NULL;
static gpointer gee_hash_map_value_collection_parent_class = NULL;
static gpointer gee_hash_map_entry_set_parent_class = NULL;
static gpointer gee_hash_map_node_iterator_parent_class = NULL;
static gpointer gee_hash_map_key_iterator_parent_class = NULL;
static GeeTraversableIface* gee_hash_map_key_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_hash_map_key_iterator_gee_iterator_parent_iface = NULL;
static gpointer gee_hash_map_map_iterator_parent_class = NULL;
static GeeMapIteratorIface* gee_hash_map_map_iterator_gee_map_iterator_parent_iface = NULL;
static gpointer gee_hash_map_value_iterator_parent_class = NULL;
static GeeTraversableIface* gee_hash_map_value_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_hash_map_value_iterator_gee_iterator_parent_iface = NULL;
static gpointer gee_hash_map_entry_iterator_parent_class = NULL;
static GeeTraversableIface* gee_hash_map_entry_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_hash_map_entry_iterator_gee_iterator_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_lazy (GValue* value, gpointer v_object);
void gee_value_take_lazy (GValue* value, gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_map_iterator_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
GType gee_set_get_type (void) G_GNUC_CONST;
GType gee_map_entry_get_type (void) G_GNUC_CONST;
GType gee_map_get_type (void) G_GNUC_CONST;
GType gee_abstract_map_get_type (void) G_GNUC_CONST;
GType gee_hash_map_get_type (void) G_GNUC_CONST;
static void gee_hash_map_node_free (GeeHashMapNode* self);
G_GNUC_INTERNAL gpointer gee_functions_hash_data_func_closure_ref (gpointer instance);
G_GNUC_INTERNAL void gee_functions_hash_data_func_closure_unref (gpointer instance);
G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_hash_data_func_closure (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
G_GNUC_INTERNAL void gee_functions_value_set_hash_data_func_closure (GValue* value, gpointer v_object) G_GNUC_UNUSED;
G_GNUC_INTERNAL void gee_functions_value_take_hash_data_func_closure (GValue* value, gpointer v_object);
G_GNUC_INTERNAL gpointer gee_functions_value_get_hash_data_func_closure (const GValue* value) G_GNUC_UNUSED;
G_GNUC_INTERNAL GType gee_functions_hash_data_func_closure_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
G_GNUC_INTERNAL gpointer gee_functions_equal_data_func_closure_ref (gpointer instance);
G_GNUC_INTERNAL void gee_functions_equal_data_func_closure_unref (gpointer instance);
G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_equal_data_func_closure (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
G_GNUC_INTERNAL void gee_functions_value_set_equal_data_func_closure (GValue* value, gpointer v_object) G_GNUC_UNUSED;
G_GNUC_INTERNAL void gee_functions_value_take_equal_data_func_closure (GValue* value, gpointer v_object);
G_GNUC_INTERNAL gpointer gee_functions_value_get_equal_data_func_closure (const GValue* value) G_GNUC_UNUSED;
G_GNUC_INTERNAL GType gee_functions_equal_data_func_closure_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define GEE_HASH_MAP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_HASH_MAP, GeeHashMapPrivate))
enum  {
	GEE_HASH_MAP_DUMMY_PROPERTY,
	GEE_HASH_MAP_K_TYPE,
	GEE_HASH_MAP_K_DUP_FUNC,
	GEE_HASH_MAP_K_DESTROY_FUNC,
	GEE_HASH_MAP_V_TYPE,
	GEE_HASH_MAP_V_DUP_FUNC,
	GEE_HASH_MAP_V_DESTROY_FUNC,
	GEE_HASH_MAP_SIZE,
	GEE_HASH_MAP_READ_ONLY,
	GEE_HASH_MAP_KEYS,
	GEE_HASH_MAP_VALUES,
	GEE_HASH_MAP_ENTRIES
};
void gee_abstract_map_clear (GeeAbstractMap* self);
#define GEE_HASH_MAP_MIN_SIZE 11
#define GEE_HASH_MAP_MAX_SIZE 13845163
GeeHashMap* gee_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashDataFunc key_hash_func, void* key_hash_func_target, GDestroyNotify key_hash_func_target_destroy_notify, GeeEqualDataFunc key_equal_func, void* key_equal_func_target, GDestroyNotify key_equal_func_target_destroy_notify, GeeEqualDataFunc value_equal_func, void* value_equal_func_target, GDestroyNotify value_equal_func_target_destroy_notify);
GeeHashMap* gee_hash_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashDataFunc key_hash_func, void* key_hash_func_target, GDestroyNotify key_hash_func_target_destroy_notify, GeeEqualDataFunc key_equal_func, void* key_equal_func_target, GDestroyNotify key_equal_func_target_destroy_notify, GeeEqualDataFunc value_equal_func, void* value_equal_func_target, GDestroyNotify value_equal_func_target_destroy_notify);
GeeAbstractMap* gee_abstract_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GeeHashDataFunc gee_functions_get_hash_func_for (GType t, void** result_target, GDestroyNotify* result_target_destroy_notify);
GeeEqualDataFunc gee_functions_get_equal_func_for (GType t, void** result_target, GDestroyNotify* result_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_functions_hash_data_func_closure_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_functions_hash_data_func_closure_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
G_GNUC_INTERNAL GeeHashMap* gee_hash_map_new_with_closures (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeFunctionsHashDataFuncClosure* key_hash_func, GeeFunctionsEqualDataFuncClosure* key_equal_func, GeeFunctionsEqualDataFuncClosure* value_equal_func);
G_GNUC_INTERNAL GeeHashMap* gee_hash_map_construct_with_closures (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeFunctionsHashDataFuncClosure* key_hash_func, GeeFunctionsEqualDataFuncClosure* key_equal_func, GeeFunctionsEqualDataFuncClosure* value_equal_func);
static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointer key);
GeeHashDataFunc gee_hash_map_get_key_hash_func (GeeHashMap* self, gpointer* result_target);
GeeEqualDataFunc gee_hash_map_get_key_equal_func (GeeHashMap* self, gpointer* result_target);
static gboolean gee_hash_map_real_has_key (GeeAbstractMap* base, gconstpointer key);
static gboolean gee_hash_map_real_has (GeeAbstractMap* base, gconstpointer key, gconstpointer value);
GeeEqualDataFunc gee_hash_map_get_value_equal_func (GeeHashMap* self, gpointer* result_target);
static gpointer gee_hash_map_real_get (GeeAbstractMap* base, gconstpointer key);
static void gee_hash_map_real_set (GeeAbstractMap* base, gconstpointer key, gconstpointer value);
static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
static inline void gee_hash_map_resize (GeeHashMap* self);
static gboolean gee_hash_map_real_unset (GeeAbstractMap* base, gconstpointer key, gpointer* value);
static inline gboolean gee_hash_map_unset_helper (GeeHashMap* self, gconstpointer key, gpointer* value);
static void gee_hash_map_real_clear (GeeAbstractMap* base);
static GeeMapIterator* gee_hash_map_real_map_iterator (GeeAbstractMap* base);
static GeeHashMapMapIterator* gee_hash_map_map_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapMapIterator* gee_hash_map_map_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_node_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GType gee_hash_map_map_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_hash_map_get_key_hash_func_closure (GeeHashMap* self);
G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_hash_map_get_key_equal_func_closure (GeeHashMap* self);
static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
GType gee_abstract_collection_get_type (void) G_GNUC_CONST;
GType gee_abstract_set_get_type (void) G_GNUC_CONST;
static GType gee_hash_map_key_set_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_value_collection_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GeeHashMapEntrySet* gee_hash_map_entry_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapEntrySet* gee_hash_map_entry_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_entry_set_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static void gee_hash_map_set_key_hash_func (GeeHashMap* self, GeeHashDataFunc value, gpointer value_target);
static void gee_hash_map_set_key_equal_func (GeeHashMap* self, GeeEqualDataFunc value, gpointer value_target);
static void gee_hash_map_set_value_equal_func (GeeHashMap* self, GeeEqualDataFunc value, gpointer value_target);
static void gee_hash_map_node_instance_init (GeeHashMapNode * self);
static GType gee_hash_map_entry_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define GEE_HASH_MAP_ENTRY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntryPrivate))
enum  {
	GEE_HASH_MAP_ENTRY_DUMMY_PROPERTY,
	GEE_HASH_MAP_ENTRY_K_TYPE,
	GEE_HASH_MAP_ENTRY_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_V_TYPE,
	GEE_HASH_MAP_ENTRY_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_KEY,
	GEE_HASH_MAP_ENTRY_VALUE,
	GEE_HASH_MAP_ENTRY_READ_ONLY
};
static GeeMapEntry* gee_hash_map_entry_entry_for (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node);
static GeeHashMapEntry* gee_hash_map_entry_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node);
static GeeHashMapEntry* gee_hash_map_entry_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node);
GeeMapEntry* gee_map_entry_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
static void gee_hash_map_entry_finalize (GObject* obj);
gconstpointer gee_map_entry_get_key (GeeMapEntry* self);
gconstpointer gee_map_entry_get_value (GeeMapEntry* self);
gboolean gee_map_entry_get_read_only (GeeMapEntry* self);
static void _vala_gee_hash_map_entry_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
void gee_map_entry_set_value (GeeMapEntry* self, gconstpointer value);
static void _vala_gee_hash_map_entry_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_KEY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetPrivate))
enum  {
	GEE_HASH_MAP_KEY_SET_DUMMY_PROPERTY,
	GEE_HASH_MAP_KEY_SET_K_TYPE,
	GEE_HASH_MAP_KEY_SET_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_SET_V_TYPE,
	GEE_HASH_MAP_KEY_SET_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_SET_SIZE,
	GEE_HASH_MAP_KEY_SET_READ_ONLY
};
GeeAbstractSet* gee_abstract_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
static GeeIterator* gee_hash_map_key_set_real_iterator (GeeAbstractCollection* base);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_key_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static gboolean gee_hash_map_key_set_real_add (GeeAbstractCollection* base, gconstpointer key);
static void gee_hash_map_key_set_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_key_set_real_remove (GeeAbstractCollection* base, gconstpointer key);
static gboolean gee_hash_map_key_set_real_contains (GeeAbstractCollection* base, gconstpointer key);
gboolean gee_abstract_map_has_key (GeeAbstractMap* self, gconstpointer key);
gint gee_abstract_map_get_size (GeeAbstractMap* self);
static void gee_hash_map_key_set_finalize (GObject* obj);
gint gee_abstract_collection_get_size (GeeAbstractCollection* self);
gboolean gee_abstract_collection_get_read_only (GeeAbstractCollection* self);
static void _vala_gee_hash_map_key_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_key_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_VALUE_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionPrivate))
enum  {
	GEE_HASH_MAP_VALUE_COLLECTION_DUMMY_PROPERTY,
	GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_SIZE,
	GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY
};
GeeAbstractCollection* gee_abstract_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeAbstractCollection* base);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_value_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static gboolean gee_hash_map_value_collection_real_add (GeeAbstractCollection* base, gconstpointer value);
static void gee_hash_map_value_collection_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_value_collection_real_remove (GeeAbstractCollection* base, gconstpointer value);
static gboolean gee_hash_map_value_collection_real_contains (GeeAbstractCollection* base, gconstpointer value);
GeeIterator* gee_abstract_collection_iterator (GeeAbstractCollection* self);
gboolean gee_iterator_next (GeeIterator* self);
gpointer gee_iterator_get (GeeIterator* self);
static void gee_hash_map_value_collection_finalize (GObject* obj);
static void _vala_gee_hash_map_value_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_value_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_ENTRY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySetPrivate))
enum  {
	GEE_HASH_MAP_ENTRY_SET_DUMMY_PROPERTY,
	GEE_HASH_MAP_ENTRY_SET_K_TYPE,
	GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_SET_V_TYPE,
	GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_SET_SIZE,
	GEE_HASH_MAP_ENTRY_SET_READ_ONLY
};
static GeeIterator* gee_hash_map_entry_set_real_iterator (GeeAbstractCollection* base);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_entry_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static gboolean gee_hash_map_entry_set_real_add (GeeAbstractCollection* base, GeeMapEntry* entry);
static void gee_hash_map_entry_set_real_clear (GeeAbstractCollection* base);
static gboolean gee_hash_map_entry_set_real_remove (GeeAbstractCollection* base, GeeMapEntry* entry);
static gboolean gee_hash_map_entry_set_real_contains (GeeAbstractCollection* base, GeeMapEntry* entry);
gboolean gee_abstract_map_has (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
static void gee_hash_map_entry_set_finalize (GObject* obj);
static void _vala_gee_hash_map_entry_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_entry_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_NODE_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIteratorPrivate))
enum  {
	GEE_HASH_MAP_NODE_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_NODE_ITERATOR_K_TYPE,
	GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_V_TYPE,
	GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY,
	GEE_HASH_MAP_NODE_ITERATOR_VALID
};
static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNodeIterator* iter);
static gboolean gee_hash_map_node_iterator_next (GeeHashMapNodeIterator* self);
static gboolean gee_hash_map_node_iterator_has_next (GeeHashMapNodeIterator* self);
static gboolean gee_hash_map_node_iterator_get_read_only (GeeHashMapNodeIterator* self);
static gboolean gee_hash_map_node_iterator_get_valid (GeeHashMapNodeIterator* self);
static void gee_hash_map_node_iterator_finalize (GObject* obj);
static void _vala_gee_hash_map_node_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_node_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_KEY_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorPrivate))
enum  {
	GEE_HASH_MAP_KEY_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_KEY_ITERATOR_K_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC
};
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapKeyIterator* iter);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapKeyIterator* iter);
static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base);
static void gee_hash_map_key_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_key_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
static GeeIterator** gee_hash_map_key_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1);
static void _vala_gee_hash_map_key_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_key_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_MAP_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIteratorPrivate))
enum  {
	GEE_HASH_MAP_MAP_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_MAP_ITERATOR_K_TYPE,
	GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_V_TYPE,
	GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC,
	GEE_HASH_MAP_MAP_ITERATOR_MUTABLE,
	GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY
};
static gpointer gee_hash_map_map_iterator_real_get_key (GeeMapIterator* base);
static void gee_hash_map_map_iterator_real_unset (GeeMapIterator* base);
gboolean gee_map_iterator_has_next (GeeMapIterator* self);
static gpointer gee_hash_map_map_iterator_real_get_value (GeeMapIterator* base);
static void gee_hash_map_map_iterator_real_set_value (GeeMapIterator* base, gconstpointer value);
void gee_abstract_map_set (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
gboolean gee_map_iterator_get_mutable (GeeMapIterator* self);
static void _vala_gee_hash_map_map_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_map_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_VALUE_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorPrivate))
enum  {
	GEE_HASH_MAP_VALUE_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC
};
static GeeHashMapValueIterator* gee_hash_map_value_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapValueIterator* iter);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapValueIterator* iter);
static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base);
static void gee_hash_map_value_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_value_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
static GeeIterator** gee_hash_map_value_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1);
static void _vala_gee_hash_map_value_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_value_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define GEE_HASH_MAP_ENTRY_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIteratorPrivate))
enum  {
	GEE_HASH_MAP_ENTRY_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC
};
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapEntryIterator* iter);
static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapEntryIterator* iter);
static GeeMapEntry* gee_hash_map_entry_iterator_real_get (GeeIterator* base);
static void gee_hash_map_entry_iterator_real_remove (GeeIterator* base);
static gboolean gee_hash_map_entry_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
static GeeIterator** gee_hash_map_entry_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1);
static void _vala_gee_hash_map_entry_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_entry_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void gee_hash_map_finalize (GObject* obj);
gboolean gee_abstract_map_get_read_only (GeeAbstractMap* self);
GeeSet* gee_abstract_map_get_keys (GeeAbstractMap* self);
GeeCollection* gee_abstract_map_get_values (GeeAbstractMap* self);
GeeSet* gee_abstract_map_get_entries (GeeAbstractMap* self);
static void _vala_gee_hash_map_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_hash_map_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


/**
 * Constructs a new, empty hash map.
 *
 * If not provided, the functions parameters are requested to the
 * {@link Functions} function factory methods.
 *
 * @param key_hash_func an optional key hash function
 * @param key_equal_func an optional key equality testing function
 * @param value_equal_func an optional value equality testing function
 */
GeeHashMap* gee_hash_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashDataFunc key_hash_func, void* key_hash_func_target, GDestroyNotify key_hash_func_target_destroy_notify, GeeEqualDataFunc key_equal_func, void* key_equal_func_target, GDestroyNotify key_equal_func_target_destroy_notify, GeeEqualDataFunc value_equal_func, void* value_equal_func_target, GDestroyNotify value_equal_func_target_destroy_notify) {
	GeeHashMap * self = NULL;
	GeeHashDataFunc _tmp0_ = NULL;
	void* _tmp0__target = NULL;
	GeeEqualDataFunc _tmp4_ = NULL;
	void* _tmp4__target = NULL;
	GeeEqualDataFunc _tmp8_ = NULL;
	void* _tmp8__target = NULL;
	GeeHashDataFunc _tmp12_ = NULL;
	void* _tmp12__target = NULL;
	GDestroyNotify _tmp12__target_destroy_notify = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp13_ = NULL;
	GeeEqualDataFunc _tmp14_ = NULL;
	void* _tmp14__target = NULL;
	GDestroyNotify _tmp14__target_destroy_notify = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp15_ = NULL;
	GeeEqualDataFunc _tmp16_ = NULL;
	void* _tmp16__target = NULL;
	GDestroyNotify _tmp16__target_destroy_notify = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	GeeHashMapNode** _tmp19_ = NULL;
	self = (GeeHashMap*) gee_abstract_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = key_hash_func;
	_tmp0__target = key_hash_func_target;
	if (_tmp0_ == NULL) {
		void* _tmp1_ = NULL;
		GDestroyNotify _tmp2_ = NULL;
		GeeHashDataFunc _tmp3_ = NULL;
		_tmp3_ = gee_functions_get_hash_func_for (k_type, &_tmp1_, &_tmp2_);
		(key_hash_func_target_destroy_notify == NULL) ? NULL : (key_hash_func_target_destroy_notify (key_hash_func_target), NULL);
		key_hash_func = NULL;
		key_hash_func_target = NULL;
		key_hash_func_target_destroy_notify = NULL;
		key_hash_func = _tmp3_;
		key_hash_func_target = _tmp1_;
		key_hash_func_target_destroy_notify = _tmp2_;
	}
	_tmp4_ = key_equal_func;
	_tmp4__target = key_equal_func_target;
	if (_tmp4_ == NULL) {
		void* _tmp5_ = NULL;
		GDestroyNotify _tmp6_ = NULL;
		GeeEqualDataFunc _tmp7_ = NULL;
		_tmp7_ = gee_functions_get_equal_func_for (k_type, &_tmp5_, &_tmp6_);
		(key_equal_func_target_destroy_notify == NULL) ? NULL : (key_equal_func_target_destroy_notify (key_equal_func_target), NULL);
		key_equal_func = NULL;
		key_equal_func_target = NULL;
		key_equal_func_target_destroy_notify = NULL;
		key_equal_func = _tmp7_;
		key_equal_func_target = _tmp5_;
		key_equal_func_target_destroy_notify = _tmp6_;
	}
	_tmp8_ = value_equal_func;
	_tmp8__target = value_equal_func_target;
	if (_tmp8_ == NULL) {
		void* _tmp9_ = NULL;
		GDestroyNotify _tmp10_ = NULL;
		GeeEqualDataFunc _tmp11_ = NULL;
		_tmp11_ = gee_functions_get_equal_func_for (v_type, &_tmp9_, &_tmp10_);
		(value_equal_func_target_destroy_notify == NULL) ? NULL : (value_equal_func_target_destroy_notify (value_equal_func_target), NULL);
		value_equal_func = NULL;
		value_equal_func_target = NULL;
		value_equal_func_target_destroy_notify = NULL;
		value_equal_func = _tmp11_;
		value_equal_func_target = _tmp9_;
		value_equal_func_target_destroy_notify = _tmp10_;
	}
	_tmp12_ = key_hash_func;
	_tmp12__target = key_hash_func_target;
	_tmp12__target_destroy_notify = key_hash_func_target_destroy_notify;
	key_hash_func_target_destroy_notify = NULL;
	_tmp13_ = gee_functions_hash_data_func_closure_new (k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, _tmp12_, _tmp12__target, _tmp12__target_destroy_notify);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	self->priv->_key_hash_func = _tmp13_;
	_tmp14_ = key_equal_func;
	_tmp14__target = key_equal_func_target;
	_tmp14__target_destroy_notify = key_equal_func_target_destroy_notify;
	key_equal_func_target_destroy_notify = NULL;
	_tmp15_ = gee_functions_equal_data_func_closure_new (k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, _tmp14_, _tmp14__target, _tmp14__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	self->priv->_key_equal_func = _tmp15_;
	_tmp16_ = value_equal_func;
	_tmp16__target = value_equal_func_target;
	_tmp16__target_destroy_notify = value_equal_func_target_destroy_notify;
	value_equal_func_target_destroy_notify = NULL;
	_tmp17_ = gee_functions_equal_data_func_closure_new (v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp16_, _tmp16__target, _tmp16__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	self->priv->_value_equal_func = _tmp17_;
	self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
	_tmp18_ = self->priv->_array_size;
	_tmp19_ = g_new0 (GeeHashMapNode*, _tmp18_ + 1);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	self->priv->_nodes = _tmp19_;
	self->priv->_nodes_length1 = _tmp18_;
	self->priv->__nodes_size_ = self->priv->_nodes_length1;
	(key_hash_func_target_destroy_notify == NULL) ? NULL : (key_hash_func_target_destroy_notify (key_hash_func_target), NULL);
	key_hash_func = NULL;
	key_hash_func_target = NULL;
	key_hash_func_target_destroy_notify = NULL;
	(key_equal_func_target_destroy_notify == NULL) ? NULL : (key_equal_func_target_destroy_notify (key_equal_func_target), NULL);
	key_equal_func = NULL;
	key_equal_func_target = NULL;
	key_equal_func_target_destroy_notify = NULL;
	(value_equal_func_target_destroy_notify == NULL) ? NULL : (value_equal_func_target_destroy_notify (value_equal_func_target), NULL);
	value_equal_func = NULL;
	value_equal_func_target = NULL;
	value_equal_func_target_destroy_notify = NULL;
	return self;
}


GeeHashMap* gee_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashDataFunc key_hash_func, void* key_hash_func_target, GDestroyNotify key_hash_func_target_destroy_notify, GeeEqualDataFunc key_equal_func, void* key_equal_func_target, GDestroyNotify key_equal_func_target_destroy_notify, GeeEqualDataFunc value_equal_func, void* value_equal_func_target, GDestroyNotify value_equal_func_target_destroy_notify) {
	return gee_hash_map_construct (GEE_TYPE_HASH_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_hash_func, key_hash_func_target, key_hash_func_target_destroy_notify, key_equal_func, key_equal_func_target, key_equal_func_target_destroy_notify, value_equal_func, value_equal_func_target, value_equal_func_target_destroy_notify);
}


static gpointer _gee_functions_hash_data_func_closure_ref0 (gpointer self) {
	return self ? gee_functions_hash_data_func_closure_ref (self) : NULL;
}


static gpointer _gee_functions_equal_data_func_closure_ref0 (gpointer self) {
	return self ? gee_functions_equal_data_func_closure_ref (self) : NULL;
}


G_GNUC_INTERNAL GeeHashMap* gee_hash_map_construct_with_closures (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeFunctionsHashDataFuncClosure* key_hash_func, GeeFunctionsEqualDataFuncClosure* key_equal_func, GeeFunctionsEqualDataFuncClosure* value_equal_func) {
	GeeHashMap * self = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp0_ = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp1_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp2_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp3_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp4_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	GeeHashMapNode** _tmp7_ = NULL;
	g_return_val_if_fail (key_hash_func != NULL, NULL);
	g_return_val_if_fail (key_equal_func != NULL, NULL);
	g_return_val_if_fail (value_equal_func != NULL, NULL);
	self = (GeeHashMap*) gee_abstract_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = key_hash_func;
	_tmp1_ = _gee_functions_hash_data_func_closure_ref0 (_tmp0_);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	self->priv->_key_hash_func = _tmp1_;
	_tmp2_ = key_equal_func;
	_tmp3_ = _gee_functions_equal_data_func_closure_ref0 (_tmp2_);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	self->priv->_key_equal_func = _tmp3_;
	_tmp4_ = value_equal_func;
	_tmp5_ = _gee_functions_equal_data_func_closure_ref0 (_tmp4_);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	self->priv->_value_equal_func = _tmp5_;
	self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
	_tmp6_ = self->priv->_array_size;
	_tmp7_ = g_new0 (GeeHashMapNode*, _tmp6_ + 1);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	self->priv->_nodes = _tmp7_;
	self->priv->_nodes_length1 = _tmp6_;
	self->priv->__nodes_size_ = self->priv->_nodes_length1;
	_gee_functions_hash_data_func_closure_unref0 (key_hash_func);
	_gee_functions_equal_data_func_closure_unref0 (key_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (value_equal_func);
	return self;
}


G_GNUC_INTERNAL GeeHashMap* gee_hash_map_new_with_closures (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeFunctionsHashDataFuncClosure* key_hash_func, GeeFunctionsEqualDataFuncClosure* key_equal_func, GeeFunctionsEqualDataFuncClosure* value_equal_func) {
	return gee_hash_map_construct_with_closures (GEE_TYPE_HASH_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_hash_func, key_equal_func, value_equal_func);
}


static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointer key) {
	GeeHashMapNode** result = NULL;
	guint hash_value = 0U;
	GeeHashDataFunc _tmp0_ = NULL;
	void* _tmp0__target = NULL;
	GeeHashDataFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	gconstpointer _tmp2_ = NULL;
	guint _tmp3_ = 0U;
	GeeHashMapNode** node = NULL;
	GeeHashMapNode** _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	guint _tmp5_ = 0U;
	gint _tmp6_ = 0;
	GeeHashMapNode** _tmp20_ = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = gee_hash_map_get_key_hash_func (self, &_tmp0__target);
	_tmp1_ = _tmp0_;
	_tmp1__target = _tmp0__target;
	_tmp2_ = key;
	_tmp3_ = _tmp1_ (_tmp2_, _tmp1__target);
	hash_value = _tmp3_;
	_tmp4_ = self->priv->_nodes;
	_tmp4__length1 = self->priv->_nodes_length1;
	_tmp5_ = hash_value;
	_tmp6_ = self->priv->_array_size;
	node = &_tmp4_[_tmp5_ % _tmp6_];
	while (TRUE) {
		gboolean _tmp7_ = FALSE;
		GeeHashMapNode** _tmp8_ = NULL;
		GeeHashMapNode** _tmp19_ = NULL;
		_tmp8_ = node;
		if ((*_tmp8_) != NULL) {
			gboolean _tmp9_ = FALSE;
			guint _tmp10_ = 0U;
			GeeHashMapNode** _tmp11_ = NULL;
			guint _tmp12_ = 0U;
			_tmp10_ = hash_value;
			_tmp11_ = node;
			_tmp12_ = (*_tmp11_)->key_hash;
			if (_tmp10_ != _tmp12_) {
				_tmp9_ = TRUE;
			} else {
				GeeEqualDataFunc _tmp13_ = NULL;
				void* _tmp13__target = NULL;
				GeeEqualDataFunc _tmp14_ = NULL;
				void* _tmp14__target = NULL;
				GeeHashMapNode** _tmp15_ = NULL;
				gconstpointer _tmp16_ = NULL;
				gconstpointer _tmp17_ = NULL;
				gboolean _tmp18_ = FALSE;
				_tmp13_ = gee_hash_map_get_key_equal_func (self, &_tmp13__target);
				_tmp14_ = _tmp13_;
				_tmp14__target = _tmp13__target;
				_tmp15_ = node;
				_tmp16_ = (*_tmp15_)->key;
				_tmp17_ = key;
				_tmp18_ = _tmp14_ (_tmp16_, _tmp17_, _tmp14__target);
				_tmp9_ = !_tmp18_;
			}
			_tmp7_ = _tmp9_;
		} else {
			_tmp7_ = FALSE;
		}
		if (!_tmp7_) {
			break;
		}
		_tmp19_ = node;
		node = &(*_tmp19_)->next;
	}
	_tmp20_ = node;
	result = _tmp20_;
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_hash_map_real_has_key (GeeAbstractMap* base, gconstpointer key) {
	GeeHashMap * self;
	gboolean result = FALSE;
	GeeHashMapNode** node = NULL;
	gconstpointer _tmp0_ = NULL;
	GeeHashMapNode** _tmp1_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = key;
	_tmp1_ = gee_hash_map_lookup_node (self, _tmp0_);
	node = _tmp1_;
	result = (*node) != NULL;
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_hash_map_real_has (GeeAbstractMap* base, gconstpointer key, gconstpointer value) {
	GeeHashMap * self;
	gboolean result = FALSE;
	GeeHashMapNode** node = NULL;
	gconstpointer _tmp0_ = NULL;
	GeeHashMapNode** _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	GeeHashMapNode** _tmp3_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = key;
	_tmp1_ = gee_hash_map_lookup_node (self, _tmp0_);
	node = _tmp1_;
	_tmp3_ = node;
	if ((*_tmp3_) != NULL) {
		GeeEqualDataFunc _tmp4_ = NULL;
		void* _tmp4__target = NULL;
		GeeEqualDataFunc _tmp5_ = NULL;
		void* _tmp5__target = NULL;
		GeeHashMapNode** _tmp6_ = NULL;
		gconstpointer _tmp7_ = NULL;
		gconstpointer _tmp8_ = NULL;
		gboolean _tmp9_ = FALSE;
		_tmp4_ = gee_hash_map_get_value_equal_func (self, &_tmp4__target);
		_tmp5_ = _tmp4_;
		_tmp5__target = _tmp4__target;
		_tmp6_ = node;
		_tmp7_ = (*_tmp6_)->value;
		_tmp8_ = value;
		_tmp9_ = _tmp5_ (_tmp7_, _tmp8_, _tmp5__target);
		_tmp2_ = _tmp9_;
	} else {
		_tmp2_ = FALSE;
	}
	result = _tmp2_;
	return result;
}


/**
 * {@inheritDoc}
 */
static gpointer gee_hash_map_real_get (GeeAbstractMap* base, gconstpointer key) {
	GeeHashMap * self;
	gpointer result = NULL;
	GeeHashMapNode* node = NULL;
	gconstpointer _tmp0_ = NULL;
	GeeHashMapNode** _tmp1_ = NULL;
	GeeHashMapNode* _tmp2_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = key;
	_tmp1_ = gee_hash_map_lookup_node (self, _tmp0_);
	node = *_tmp1_;
	_tmp2_ = node;
	if (_tmp2_ != NULL) {
		GeeHashMapNode* _tmp3_ = NULL;
		gconstpointer _tmp4_ = NULL;
		gpointer _tmp5_ = NULL;
		_tmp3_ = node;
		_tmp4_ = _tmp3_->value;
		_tmp5_ = ((_tmp4_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp4_) : ((gpointer) _tmp4_);
		result = _tmp5_;
		return result;
	} else {
		result = NULL;
		return result;
	}
}


/**
 * {@inheritDoc}
 */
static void gee_hash_map_real_set (GeeAbstractMap* base, gconstpointer key, gconstpointer value) {
	GeeHashMap * self;
	GeeHashMapNode** node = NULL;
	gconstpointer _tmp0_ = NULL;
	GeeHashMapNode** _tmp1_ = NULL;
	GeeHashMapNode** _tmp2_ = NULL;
	gint _tmp19_ = 0;
	self = (GeeHashMap*) base;
	_tmp0_ = key;
	_tmp1_ = gee_hash_map_lookup_node (self, _tmp0_);
	node = _tmp1_;
	_tmp2_ = node;
	if ((*_tmp2_) != NULL) {
		GeeHashMapNode** _tmp3_ = NULL;
		gconstpointer _tmp4_ = NULL;
		gpointer _tmp5_ = NULL;
		_tmp3_ = node;
		_tmp4_ = value;
		_tmp5_ = ((_tmp4_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp4_) : ((gpointer) _tmp4_);
		(((*_tmp3_)->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : ((*_tmp3_)->value = (self->priv->v_destroy_func ((*_tmp3_)->value), NULL));
		(*_tmp3_)->value = _tmp5_;
	} else {
		guint hash_value = 0U;
		GeeHashDataFunc _tmp6_ = NULL;
		void* _tmp6__target = NULL;
		GeeHashDataFunc _tmp7_ = NULL;
		void* _tmp7__target = NULL;
		gconstpointer _tmp8_ = NULL;
		guint _tmp9_ = 0U;
		GeeHashMapNode** _tmp10_ = NULL;
		gconstpointer _tmp11_ = NULL;
		gpointer _tmp12_ = NULL;
		gconstpointer _tmp13_ = NULL;
		gpointer _tmp14_ = NULL;
		guint _tmp15_ = 0U;
		GeeHashMapNode* _tmp16_ = NULL;
		GeeHashMapNode* _tmp17_ = NULL;
		gint _tmp18_ = 0;
		_tmp6_ = gee_hash_map_get_key_hash_func (self, &_tmp6__target);
		_tmp7_ = _tmp6_;
		_tmp7__target = _tmp6__target;
		_tmp8_ = key;
		_tmp9_ = _tmp7_ (_tmp8_, _tmp7__target);
		hash_value = _tmp9_;
		_tmp10_ = node;
		_tmp11_ = key;
		_tmp12_ = ((_tmp11_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp11_) : ((gpointer) _tmp11_);
		_tmp13_ = value;
		_tmp14_ = ((_tmp13_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp13_) : ((gpointer) _tmp13_);
		_tmp15_ = hash_value;
		_tmp16_ = gee_hash_map_node_new (_tmp12_, _tmp14_, _tmp15_);
		*_tmp10_ = _tmp16_;
		_tmp17_ = *_tmp10_;
		_tmp18_ = self->priv->_nnodes;
		self->priv->_nnodes = _tmp18_ + 1;
		gee_hash_map_resize (self);
	}
	_tmp19_ = self->priv->_stamp;
	self->priv->_stamp = _tmp19_ + 1;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_hash_map_real_unset (GeeAbstractMap* base, gconstpointer key, gpointer* value) {
	GeeHashMap * self;
	gpointer _vala_value = NULL;
	gboolean result = FALSE;
	gboolean b = FALSE;
	gconstpointer _tmp0_ = NULL;
	gpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	self = (GeeHashMap*) base;
	_tmp0_ = key;
	_tmp2_ = gee_hash_map_unset_helper (self, _tmp0_, &_tmp1_);
	((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	_vala_value = _tmp1_;
	b = _tmp2_;
	_tmp3_ = b;
	if (_tmp3_) {
		gee_hash_map_resize (self);
	}
	result = b;
	if (value) {
		*value = _vala_value;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	}
	return result;
}


/**
 * {@inheritDoc}
 */
static void gee_hash_map_real_clear (GeeAbstractMap* base) {
	GeeHashMap * self;
	self = (GeeHashMap*) base;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				GeeHashMapNode* node = NULL;
				GeeHashMapNode** _tmp4_ = NULL;
				gint _tmp4__length1 = 0;
				gint _tmp5_ = 0;
				GeeHashMapNode* _tmp6_ = NULL;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = i;
					i = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = i;
				_tmp3_ = self->priv->_array_size;
				if (!(_tmp2_ < _tmp3_)) {
					break;
				}
				_tmp4_ = self->priv->_nodes;
				_tmp4__length1 = self->priv->_nodes_length1;
				_tmp5_ = i;
				_tmp6_ = _tmp4_[_tmp5_];
				_tmp4_[_tmp5_] = NULL;
				node = _tmp6_;
				while (TRUE) {
					GeeHashMapNode* _tmp7_ = NULL;
					GeeHashMapNode* next = NULL;
					GeeHashMapNode* _tmp8_ = NULL;
					GeeHashMapNode* _tmp9_ = NULL;
					GeeHashMapNode* _tmp10_ = NULL;
					GeeHashMapNode* _tmp11_ = NULL;
					GeeHashMapNode* _tmp12_ = NULL;
					_tmp7_ = node;
					if (!(_tmp7_ != NULL)) {
						break;
					}
					_tmp8_ = node;
					_tmp9_ = _tmp8_->next;
					_tmp8_->next = NULL;
					next = _tmp9_;
					_tmp10_ = node;
					((_tmp10_->key == NULL) || (self->priv->k_destroy_func == NULL)) ? NULL : (_tmp10_->key = (self->priv->k_destroy_func (_tmp10_->key), NULL));
					_tmp10_->key = NULL;
					_tmp11_ = node;
					((_tmp11_->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp11_->value = (self->priv->v_destroy_func (_tmp11_->value), NULL));
					_tmp11_->value = NULL;
					_tmp12_ = next;
					next = NULL;
					_gee_hash_map_node_free0 (node);
					node = _tmp12_;
					_gee_hash_map_node_free0 (next);
				}
				_gee_hash_map_node_free0 (node);
			}
		}
	}
	self->priv->_nnodes = 0;
	gee_hash_map_resize (self);
}


/**
 * {@inheritDoc}
 */
static GeeMapIterator* gee_hash_map_real_map_iterator (GeeAbstractMap* base) {
	GeeHashMap * self;
	GeeMapIterator* result = NULL;
	GeeHashMapMapIterator* _tmp0_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = gee_hash_map_map_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
	result = (GeeMapIterator*) _tmp0_;
	return result;
}


G_GNUC_INTERNAL GeeFunctionsHashDataFuncClosure* gee_hash_map_get_key_hash_func_closure (GeeHashMap* self) {
	GeeFunctionsHashDataFuncClosure* result = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp0_ = NULL;
	GeeFunctionsHashDataFuncClosure* _tmp1_ = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_hash_func;
	_tmp1_ = _gee_functions_hash_data_func_closure_ref0 (_tmp0_);
	result = _tmp1_;
	return result;
}


G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_hash_map_get_key_equal_func_closure (GeeHashMap* self) {
	GeeFunctionsEqualDataFuncClosure* result = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp0_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp1_ = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_equal_func;
	_tmp1_ = _gee_functions_equal_data_func_closure_ref0 (_tmp0_);
	result = _tmp1_;
	return result;
}


static inline gboolean gee_hash_map_unset_helper (GeeHashMap* self, gconstpointer key, gpointer* value) {
	gpointer _vala_value = NULL;
	gboolean result = FALSE;
	GeeHashMapNode** node = NULL;
	gconstpointer _tmp0_ = NULL;
	GeeHashMapNode** _tmp1_ = NULL;
	GeeHashMapNode** _tmp2_ = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = key;
	_tmp1_ = gee_hash_map_lookup_node (self, _tmp0_);
	node = _tmp1_;
	_tmp2_ = node;
	if ((*_tmp2_) != NULL) {
		GeeHashMapNode* next = NULL;
		GeeHashMapNode** _tmp3_ = NULL;
		GeeHashMapNode* _tmp4_ = NULL;
		GeeHashMapNode** _tmp5_ = NULL;
		gpointer _tmp6_ = NULL;
		GeeHashMapNode** _tmp7_ = NULL;
		GeeHashMapNode** _tmp8_ = NULL;
		GeeHashMapNode** _tmp9_ = NULL;
		GeeHashMapNode** _tmp10_ = NULL;
		GeeHashMapNode* _tmp11_ = NULL;
		GeeHashMapNode* _tmp12_ = NULL;
		gint _tmp13_ = 0;
		gint _tmp14_ = 0;
		_tmp3_ = node;
		_tmp4_ = (*_tmp3_)->next;
		(*_tmp3_)->next = NULL;
		next = _tmp4_;
		_tmp5_ = node;
		_tmp6_ = (*_tmp5_)->value;
		(*_tmp5_)->value = NULL;
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		_vala_value = _tmp6_;
		_tmp7_ = node;
		(((*_tmp7_)->key == NULL) || (self->priv->k_destroy_func == NULL)) ? NULL : ((*_tmp7_)->key = (self->priv->k_destroy_func ((*_tmp7_)->key), NULL));
		(*_tmp7_)->key = NULL;
		_tmp8_ = node;
		(((*_tmp8_)->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : ((*_tmp8_)->value = (self->priv->v_destroy_func ((*_tmp8_)->value), NULL));
		(*_tmp8_)->value = NULL;
		_tmp9_ = node;
		gee_hash_map_node_free (*_tmp9_);
		_tmp10_ = node;
		_tmp11_ = next;
		next = NULL;
		*_tmp10_ = _tmp11_;
		_tmp12_ = *_tmp10_;
		_tmp13_ = self->priv->_nnodes;
		self->priv->_nnodes = _tmp13_ - 1;
		_tmp14_ = self->priv->_stamp;
		self->priv->_stamp = _tmp14_ + 1;
		result = TRUE;
		_gee_hash_map_node_free0 (next);
		if (value) {
			*value = _vala_value;
		} else {
			((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		}
		return result;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
		_vala_value = NULL;
	}
	result = FALSE;
	if (value) {
		*value = _vala_value;
	} else {
		((_vala_value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_vala_value = (self->priv->v_destroy_func (_vala_value), NULL));
	}
	return result;
}


static inline void gee_hash_map_resize (GeeHashMap* self) {
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	g_return_if_fail (self != NULL);
	_tmp2_ = self->priv->_array_size;
	_tmp3_ = self->priv->_nnodes;
	if (_tmp2_ >= (3 * _tmp3_)) {
		gint _tmp4_ = 0;
		_tmp4_ = self->priv->_array_size;
		_tmp1_ = _tmp4_ >= GEE_HASH_MAP_MIN_SIZE;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		gboolean _tmp5_ = FALSE;
		gint _tmp6_ = 0;
		gint _tmp7_ = 0;
		_tmp6_ = self->priv->_array_size;
		_tmp7_ = self->priv->_nnodes;
		if ((3 * _tmp6_) <= _tmp7_) {
			gint _tmp8_ = 0;
			_tmp8_ = self->priv->_array_size;
			_tmp5_ = _tmp8_ < GEE_HASH_MAP_MAX_SIZE;
		} else {
			_tmp5_ = FALSE;
		}
		_tmp0_ = _tmp5_;
	}
	if (_tmp0_) {
		gint new_array_size = 0;
		gint _tmp9_ = 0;
		guint _tmp10_ = 0U;
		gint _tmp11_ = 0;
		gint _tmp12_ = 0;
		GeeHashMapNode** new_nodes = NULL;
		gint _tmp13_ = 0;
		GeeHashMapNode** _tmp14_ = NULL;
		gint new_nodes_length1 = 0;
		gint _new_nodes_size_ = 0;
		GeeHashMapNode** _tmp38_ = NULL;
		gint _tmp38__length1 = 0;
		gint _tmp39_ = 0;
		_tmp9_ = self->priv->_nnodes;
		_tmp10_ = g_spaced_primes_closest ((guint) _tmp9_);
		new_array_size = (gint) _tmp10_;
		_tmp11_ = new_array_size;
		_tmp12_ = CLAMP (_tmp11_, GEE_HASH_MAP_MIN_SIZE, GEE_HASH_MAP_MAX_SIZE);
		new_array_size = _tmp12_;
		_tmp13_ = new_array_size;
		_tmp14_ = g_new0 (GeeHashMapNode*, _tmp13_ + 1);
		new_nodes = _tmp14_;
		new_nodes_length1 = _tmp13_;
		_new_nodes_size_ = new_nodes_length1;
		{
			gint i = 0;
			i = 0;
			{
				gboolean _tmp15_ = FALSE;
				_tmp15_ = TRUE;
				while (TRUE) {
					gint _tmp17_ = 0;
					gint _tmp18_ = 0;
					GeeHashMapNode* node = NULL;
					GeeHashMapNode* next = NULL;
					if (!_tmp15_) {
						gint _tmp16_ = 0;
						_tmp16_ = i;
						i = _tmp16_ + 1;
					}
					_tmp15_ = FALSE;
					_tmp17_ = i;
					_tmp18_ = self->priv->_array_size;
					if (!(_tmp17_ < _tmp18_)) {
						break;
					}
					next = NULL;
					{
						GeeHashMapNode** _tmp19_ = NULL;
						gint _tmp19__length1 = 0;
						gint _tmp20_ = 0;
						GeeHashMapNode* _tmp21_ = NULL;
						gboolean _tmp22_ = FALSE;
						_tmp19_ = self->priv->_nodes;
						_tmp19__length1 = self->priv->_nodes_length1;
						_tmp20_ = i;
						_tmp21_ = _tmp19_[_tmp20_];
						_tmp19_[_tmp20_] = NULL;
						_gee_hash_map_node_free0 (node);
						node = _tmp21_;
						_tmp22_ = TRUE;
						while (TRUE) {
							GeeHashMapNode* _tmp24_ = NULL;
							GeeHashMapNode* _tmp25_ = NULL;
							GeeHashMapNode* _tmp26_ = NULL;
							guint hash_val = 0U;
							GeeHashMapNode* _tmp27_ = NULL;
							guint _tmp28_ = 0U;
							gint _tmp29_ = 0;
							GeeHashMapNode* _tmp30_ = NULL;
							GeeHashMapNode** _tmp31_ = NULL;
							gint _tmp31__length1 = 0;
							guint _tmp32_ = 0U;
							GeeHashMapNode* _tmp33_ = NULL;
							GeeHashMapNode** _tmp34_ = NULL;
							gint _tmp34__length1 = 0;
							guint _tmp35_ = 0U;
							GeeHashMapNode* _tmp36_ = NULL;
							GeeHashMapNode* _tmp37_ = NULL;
							if (!_tmp22_) {
								GeeHashMapNode* _tmp23_ = NULL;
								_tmp23_ = next;
								next = NULL;
								_gee_hash_map_node_free0 (node);
								node = _tmp23_;
							}
							_tmp22_ = FALSE;
							_tmp24_ = node;
							if (!(_tmp24_ != NULL)) {
								break;
							}
							_tmp25_ = node;
							_tmp26_ = _tmp25_->next;
							_tmp25_->next = NULL;
							_gee_hash_map_node_free0 (next);
							next = _tmp26_;
							_tmp27_ = node;
							_tmp28_ = _tmp27_->key_hash;
							_tmp29_ = new_array_size;
							hash_val = _tmp28_ % _tmp29_;
							_tmp30_ = node;
							_tmp31_ = new_nodes;
							_tmp31__length1 = new_nodes_length1;
							_tmp32_ = hash_val;
							_tmp33_ = _tmp31_[_tmp32_];
							_tmp31_[_tmp32_] = NULL;
							_gee_hash_map_node_free0 (_tmp30_->next);
							_tmp30_->next = _tmp33_;
							_tmp34_ = new_nodes;
							_tmp34__length1 = new_nodes_length1;
							_tmp35_ = hash_val;
							_tmp36_ = node;
							node = NULL;
							_gee_hash_map_node_free0 (_tmp34_[_tmp35_]);
							_tmp34_[_tmp35_] = _tmp36_;
							_tmp37_ = _tmp34_[_tmp35_];
						}
					}
					_gee_hash_map_node_free0 (next);
					_gee_hash_map_node_free0 (node);
				}
			}
		}
		_tmp38_ = new_nodes;
		_tmp38__length1 = new_nodes_length1;
		new_nodes = NULL;
		new_nodes_length1 = 0;
		self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
		self->priv->_nodes = _tmp38_;
		self->priv->_nodes_length1 = _tmp38__length1;
		self->priv->__nodes_size_ = self->priv->_nodes_length1;
		_tmp39_ = new_array_size;
		self->priv->_array_size = _tmp39_;
		new_nodes = (_vala_array_free (new_nodes, new_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	}
}


static gint gee_hash_map_real_get_size (GeeAbstractMap* base) {
	gint result;
	GeeHashMap* self;
	gint _tmp0_ = 0;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_nnodes;
	result = _tmp0_;
	return result;
}


static gboolean gee_hash_map_real_get_read_only (GeeAbstractMap* base) {
	gboolean result;
	GeeHashMap* self;
	self = (GeeHashMap*) base;
	result = FALSE;
	return result;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static GeeSet* gee_hash_map_real_get_keys (GeeAbstractMap* base) {
	GeeSet* result;
	GeeHashMap* self;
	GeeSet* keys = NULL;
	GeeSet* _tmp0_ = NULL;
	GeeSet* _tmp1_ = NULL;
	GeeSet* _tmp2_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_keys;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	keys = _tmp1_;
	_tmp2_ = self->priv->_keys;
	if (_tmp2_ == NULL) {
		GeeHashMapKeySet* _tmp3_ = NULL;
		GeeSet* _tmp4_ = NULL;
		GeeSet* _tmp5_ = NULL;
		_tmp3_ = gee_hash_map_key_set_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
		_g_object_unref0 (keys);
		keys = (GeeSet*) _tmp3_;
		_tmp4_ = keys;
		self->priv->_keys = _tmp4_;
		_tmp5_ = keys;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_keys));
	}
	result = keys;
	return result;
}


static GeeCollection* gee_hash_map_real_get_values (GeeAbstractMap* base) {
	GeeCollection* result;
	GeeHashMap* self;
	GeeCollection* values = NULL;
	GeeCollection* _tmp0_ = NULL;
	GeeCollection* _tmp1_ = NULL;
	GeeCollection* _tmp2_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_values;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	values = _tmp1_;
	_tmp2_ = self->priv->_values;
	if (_tmp2_ == NULL) {
		GeeHashMapValueCollection* _tmp3_ = NULL;
		GeeCollection* _tmp4_ = NULL;
		GeeCollection* _tmp5_ = NULL;
		_tmp3_ = gee_hash_map_value_collection_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
		_g_object_unref0 (values);
		values = (GeeCollection*) _tmp3_;
		_tmp4_ = values;
		self->priv->_values = _tmp4_;
		_tmp5_ = values;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_values));
	}
	result = values;
	return result;
}


static GeeSet* gee_hash_map_real_get_entries (GeeAbstractMap* base) {
	GeeSet* result;
	GeeHashMap* self;
	GeeSet* entries = NULL;
	GeeSet* _tmp0_ = NULL;
	GeeSet* _tmp1_ = NULL;
	GeeSet* _tmp2_ = NULL;
	self = (GeeHashMap*) base;
	_tmp0_ = self->priv->_entries;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	entries = _tmp1_;
	_tmp2_ = self->priv->_entries;
	if (_tmp2_ == NULL) {
		GeeHashMapEntrySet* _tmp3_ = NULL;
		GeeSet* _tmp4_ = NULL;
		GeeSet* _tmp5_ = NULL;
		_tmp3_ = gee_hash_map_entry_set_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
		_g_object_unref0 (entries);
		entries = (GeeSet*) _tmp3_;
		_tmp4_ = entries;
		self->priv->_entries = _tmp4_;
		_tmp5_ = entries;
		g_object_add_weak_pointer ((GObject*) _tmp5_, (void**) (&self->priv->_entries));
	}
	result = entries;
	return result;
}


GeeHashDataFunc gee_hash_map_get_key_hash_func (GeeHashMap* self, gpointer* result_target) {
	GeeHashDataFunc result;
	GeeFunctionsHashDataFuncClosure* _tmp0_ = NULL;
	GeeHashDataFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GeeHashDataFunc _tmp2_ = NULL;
	void* _tmp2__target = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_hash_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}


static void gee_hash_map_set_key_hash_func (GeeHashMap* self, GeeHashDataFunc value, gpointer value_target) {
	g_return_if_fail (self != NULL);
}


GeeEqualDataFunc gee_hash_map_get_key_equal_func (GeeHashMap* self, gpointer* result_target) {
	GeeEqualDataFunc result;
	GeeFunctionsEqualDataFuncClosure* _tmp0_ = NULL;
	GeeEqualDataFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GeeEqualDataFunc _tmp2_ = NULL;
	void* _tmp2__target = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_key_equal_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}


static void gee_hash_map_set_key_equal_func (GeeHashMap* self, GeeEqualDataFunc value, gpointer value_target) {
	g_return_if_fail (self != NULL);
}


GeeEqualDataFunc gee_hash_map_get_value_equal_func (GeeHashMap* self, gpointer* result_target) {
	GeeEqualDataFunc result;
	GeeFunctionsEqualDataFuncClosure* _tmp0_ = NULL;
	GeeEqualDataFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GeeEqualDataFunc _tmp2_ = NULL;
	void* _tmp2__target = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_value_equal_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}


static void gee_hash_map_set_value_equal_func (GeeHashMap* self, GeeEqualDataFunc value, gpointer value_target) {
	g_return_if_fail (self != NULL);
}


static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash) {
	GeeHashMapNode* self;
	gpointer _tmp0_ = NULL;
	gpointer _tmp1_ = NULL;
	guint _tmp2_ = 0U;
	self = g_slice_new0 (GeeHashMapNode);
	gee_hash_map_node_instance_init (self);
	_tmp0_ = k;
	k = NULL;
	self->key = _tmp0_;
	_tmp1_ = v;
	v = NULL;
	self->value = _tmp1_;
	_tmp2_ = hash;
	self->key_hash = _tmp2_;
	self->entry = NULL;
	return self;
}


static void gee_hash_map_node_instance_init (GeeHashMapNode * self) {
}


static void gee_hash_map_node_free (GeeHashMapNode* self) {
	_gee_hash_map_node_free0 (self->next);
	g_slice_free (GeeHashMapNode, self);
}


static GeeMapEntry* gee_hash_map_entry_entry_for (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node) {
	GeeMapEntry* result = NULL;
	GeeMapEntry* _result_ = NULL;
	GeeHashMapNode* _tmp0_ = NULL;
	GeeMapEntry* _tmp1_ = NULL;
	GeeMapEntry* _tmp2_ = NULL;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeMapEntry* _tmp4_ = NULL;
	g_return_val_if_fail (node != NULL, NULL);
	_tmp0_ = node;
	_tmp1_ = _tmp0_->entry;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	_result_ = _tmp2_;
	_tmp3_ = node;
	_tmp4_ = _tmp3_->entry;
	if (_tmp4_ == NULL) {
		GeeHashMapNode* _tmp5_ = NULL;
		GeeHashMapEntry* _tmp6_ = NULL;
		GeeHashMapNode* _tmp7_ = NULL;
		GeeMapEntry* _tmp8_ = NULL;
		GeeMapEntry* _tmp9_ = NULL;
		GeeHashMapNode* _tmp10_ = NULL;
		_tmp5_ = node;
		_tmp6_ = gee_hash_map_entry_new (k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp5_);
		_g_object_unref0 (_result_);
		_result_ = (GeeMapEntry*) _tmp6_;
		_tmp7_ = node;
		_tmp8_ = _result_;
		_tmp7_->entry = _tmp8_;
		_tmp9_ = _result_;
		_tmp10_ = node;
		g_object_add_weak_pointer ((GObject*) _tmp9_, (void**) (&_tmp10_->entry));
	}
	result = _result_;
	return result;
}


static GeeHashMapEntry* gee_hash_map_entry_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node) {
	GeeHashMapEntry * self = NULL;
	GeeHashMapNode* _tmp0_ = NULL;
	g_return_val_if_fail (node != NULL, NULL);
	self = (GeeHashMapEntry*) gee_map_entry_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = node;
	self->priv->_node = _tmp0_;
	return self;
}


static GeeHashMapEntry* gee_hash_map_entry_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNode* node) {
	return gee_hash_map_entry_construct (GEE_HASH_MAP_TYPE_ENTRY, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, node);
}


static gconstpointer gee_hash_map_entry_real_get_key (GeeMapEntry* base) {
	gconstpointer result;
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = _tmp0_->key;
	result = _tmp1_;
	return result;
}


static gconstpointer gee_hash_map_entry_real_get_value (GeeMapEntry* base) {
	gconstpointer result;
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = _tmp0_->value;
	result = _tmp1_;
	return result;
}


static void gee_hash_map_entry_real_set_value (GeeMapEntry* base, gconstpointer value) {
	GeeHashMapEntry* self;
	GeeHashMapNode* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gpointer _tmp2_ = NULL;
	self = (GeeHashMapEntry*) base;
	_tmp0_ = self->priv->_node;
	_tmp1_ = value;
	_tmp2_ = ((_tmp1_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp1_) : ((gpointer) _tmp1_);
	((_tmp0_->value == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp0_->value = (self->priv->v_destroy_func (_tmp0_->value), NULL));
	_tmp0_->value = _tmp2_;
	g_object_notify ((GObject *) self, "value");
}


static gboolean gee_hash_map_entry_real_get_read_only (GeeMapEntry* base) {
	gboolean result;
	GeeHashMapEntry* self;
	self = (GeeHashMapEntry*) base;
	result = FALSE;
	return result;
}


static void gee_hash_map_entry_class_init (GeeHashMapEntryClass * klass) {
	gee_hash_map_entry_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapEntryPrivate));
	GEE_MAP_ENTRY_CLASS (klass)->get_key = gee_hash_map_entry_real_get_key;
	GEE_MAP_ENTRY_CLASS (klass)->get_value = gee_hash_map_entry_real_get_value;
	GEE_MAP_ENTRY_CLASS (klass)->set_value = gee_hash_map_entry_real_set_value;
	GEE_MAP_ENTRY_CLASS (klass)->get_read_only = gee_hash_map_entry_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_entry_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_KEY, g_param_spec_pointer ("key", "key", "key", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_VALUE, g_param_spec_pointer ("value", "value", "value", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_entry_instance_init (GeeHashMapEntry * self) {
	self->priv = GEE_HASH_MAP_ENTRY_GET_PRIVATE (self);
}


static void gee_hash_map_entry_finalize (GObject* obj) {
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	G_OBJECT_CLASS (gee_hash_map_entry_parent_class)->finalize (obj);
}


static GType gee_hash_map_entry_get_type (void) {
	static volatile gsize gee_hash_map_entry_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_entry_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntry), 0, (GInstanceInitFunc) gee_hash_map_entry_instance_init, NULL };
		GType gee_hash_map_entry_type_id;
		gee_hash_map_entry_type_id = g_type_register_static (GEE_MAP_TYPE_ENTRY, "GeeHashMapEntry", &g_define_type_info, 0);
		g_once_init_leave (&gee_hash_map_entry_type_id__volatile, gee_hash_map_entry_type_id);
	}
	return gee_hash_map_entry_type_id__volatile;
}


static void _vala_gee_hash_map_entry_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_KEY:
		g_value_set_pointer (value, gee_map_entry_get_key ((GeeMapEntry*) self));
		break;
		case GEE_HASH_MAP_ENTRY_VALUE:
		g_value_set_pointer (value, gee_map_entry_get_value ((GeeMapEntry*) self));
		break;
		case GEE_HASH_MAP_ENTRY_READ_ONLY:
		g_value_set_boolean (value, gee_map_entry_get_read_only ((GeeMapEntry*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_entry_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY, GeeHashMapEntry);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_VALUE:
		gee_map_entry_set_value ((GeeMapEntry*) self, g_value_get_pointer (value));
		break;
		case GEE_HASH_MAP_ENTRY_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapKeySet * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMap* _tmp1_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapKeySet*) gee_abstract_set_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp1_;
	return self;
}


static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_key_set_construct (GEE_HASH_MAP_TYPE_KEY_SET, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeIterator* gee_hash_map_key_set_real_iterator (GeeAbstractCollection* base) {
	GeeHashMapKeySet * self;
	GeeIterator* result = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMapKeyIterator* _tmp1_ = NULL;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_key_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}


static gboolean gee_hash_map_key_set_real_add (GeeAbstractCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	gboolean result = FALSE;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
	return result;
}


static void gee_hash_map_key_set_real_clear (GeeAbstractCollection* base) {
	GeeHashMapKeySet * self;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_key_set_real_remove (GeeAbstractCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	gboolean result = FALSE;
	self = (GeeHashMapKeySet*) base;
	g_assert_not_reached ();
	return result;
}


static gboolean gee_hash_map_key_set_real_contains (GeeAbstractCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	gboolean result = FALSE;
	GeeHashMap* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = key;
	_tmp2_ = gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, _tmp1_);
	result = _tmp2_;
	return result;
}


static gint gee_hash_map_key_set_real_get_size (GeeAbstractCollection* base) {
	gint result;
	GeeHashMapKeySet* self;
	GeeHashMap* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	self = (GeeHashMapKeySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_hash_map_key_set_real_get_read_only (GeeAbstractCollection* base) {
	gboolean result;
	GeeHashMapKeySet* self;
	self = (GeeHashMapKeySet*) base;
	result = TRUE;
	return result;
}


static void gee_hash_map_key_set_class_init (GeeHashMapKeySetClass * klass) {
	gee_hash_map_key_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapKeySetPrivate));
	((GeeAbstractCollectionClass *) klass)->iterator = gee_hash_map_key_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = gee_hash_map_key_set_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = gee_hash_map_key_set_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = gee_hash_map_key_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = gee_hash_map_key_set_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_key_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_key_set_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_key_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_key_set_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_key_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_key_set_instance_init (GeeHashMapKeySet * self) {
	self->priv = GEE_HASH_MAP_KEY_SET_GET_PRIVATE (self);
}


static void gee_hash_map_key_set_finalize (GObject* obj) {
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_key_set_parent_class)->finalize (obj);
}


static GType gee_hash_map_key_set_get_type (void) {
	static volatile gsize gee_hash_map_key_set_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_key_set_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeySet), 0, (GInstanceInitFunc) gee_hash_map_key_set_instance_init, NULL };
		GType gee_hash_map_key_set_type_id;
		gee_hash_map_key_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_SET, "GeeHashMapKeySet", &g_define_type_info, 0);
		g_once_init_leave (&gee_hash_map_key_set_type_id__volatile, gee_hash_map_key_set_type_id);
	}
	return gee_hash_map_key_set_type_id__volatile;
}


static void _vala_gee_hash_map_key_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_SIZE:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_KEY_SET_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_key_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapKeySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapValueCollection * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMap* _tmp1_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapValueCollection*) gee_abstract_collection_construct (object_type, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp1_;
	return self;
}


static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_value_collection_construct (GEE_HASH_MAP_TYPE_VALUE_COLLECTION, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeAbstractCollection* base) {
	GeeHashMapValueCollection * self;
	GeeIterator* result = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMapValueIterator* _tmp1_ = NULL;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_value_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}


static gboolean gee_hash_map_value_collection_real_add (GeeAbstractCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	gboolean result = FALSE;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
	return result;
}


static void gee_hash_map_value_collection_real_clear (GeeAbstractCollection* base) {
	GeeHashMapValueCollection * self;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_value_collection_real_remove (GeeAbstractCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	gboolean result = FALSE;
	self = (GeeHashMapValueCollection*) base;
	g_assert_not_reached ();
	return result;
}


static gboolean gee_hash_map_value_collection_real_contains (GeeAbstractCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	gboolean result = FALSE;
	GeeIterator* it = NULL;
	GeeIterator* _tmp0_ = NULL;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = gee_abstract_collection_iterator ((GeeAbstractCollection*) self);
	it = _tmp0_;
	while (TRUE) {
		GeeIterator* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		GeeHashMap* _tmp3_ = NULL;
		GeeEqualDataFunc _tmp4_ = NULL;
		void* _tmp4__target = NULL;
		GeeEqualDataFunc _tmp5_ = NULL;
		void* _tmp5__target = NULL;
		GeeIterator* _tmp6_ = NULL;
		gpointer _tmp7_ = NULL;
		gpointer _tmp8_ = NULL;
		gconstpointer _tmp9_ = NULL;
		gboolean _tmp10_ = FALSE;
		gboolean _tmp11_ = FALSE;
		_tmp1_ = it;
		_tmp2_ = gee_iterator_next (_tmp1_);
		if (!_tmp2_) {
			break;
		}
		_tmp3_ = self->priv->_map;
		_tmp4_ = gee_hash_map_get_value_equal_func (_tmp3_, &_tmp4__target);
		_tmp5_ = _tmp4_;
		_tmp5__target = _tmp4__target;
		_tmp6_ = it;
		_tmp7_ = gee_iterator_get (_tmp6_);
		_tmp8_ = _tmp7_;
		_tmp9_ = value;
		_tmp10_ = _tmp5_ (_tmp8_, _tmp9_, _tmp5__target);
		_tmp11_ = _tmp10_;
		((_tmp8_ == NULL) || (self->priv->v_destroy_func == NULL)) ? NULL : (_tmp8_ = (self->priv->v_destroy_func (_tmp8_), NULL));
		if (_tmp11_) {
			result = TRUE;
			_g_object_unref0 (it);
			return result;
		}
	}
	result = FALSE;
	_g_object_unref0 (it);
	return result;
}


static gint gee_hash_map_value_collection_real_get_size (GeeAbstractCollection* base) {
	gint result;
	GeeHashMapValueCollection* self;
	GeeHashMap* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	self = (GeeHashMapValueCollection*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_hash_map_value_collection_real_get_read_only (GeeAbstractCollection* base) {
	gboolean result;
	GeeHashMapValueCollection* self;
	self = (GeeHashMapValueCollection*) base;
	result = TRUE;
	return result;
}


static void gee_hash_map_value_collection_class_init (GeeHashMapValueCollectionClass * klass) {
	gee_hash_map_value_collection_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapValueCollectionPrivate));
	((GeeAbstractCollectionClass *) klass)->iterator = gee_hash_map_value_collection_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = gee_hash_map_value_collection_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = gee_hash_map_value_collection_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = gee_hash_map_value_collection_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = gee_hash_map_value_collection_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_value_collection_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_value_collection_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_value_collection_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_value_collection_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_value_collection_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_value_collection_instance_init (GeeHashMapValueCollection * self) {
	self->priv = GEE_HASH_MAP_VALUE_COLLECTION_GET_PRIVATE (self);
}


static void gee_hash_map_value_collection_finalize (GObject* obj) {
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_value_collection_parent_class)->finalize (obj);
}


static GType gee_hash_map_value_collection_get_type (void) {
	static volatile gsize gee_hash_map_value_collection_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_value_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueCollection), 0, (GInstanceInitFunc) gee_hash_map_value_collection_instance_init, NULL };
		GType gee_hash_map_value_collection_type_id;
		gee_hash_map_value_collection_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeHashMapValueCollection", &g_define_type_info, 0);
		g_once_init_leave (&gee_hash_map_value_collection_type_id__volatile, gee_hash_map_value_collection_type_id);
	}
	return gee_hash_map_value_collection_type_id__volatile;
}


static void _vala_gee_hash_map_value_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_SIZE:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_value_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapValueCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapEntrySet* gee_hash_map_entry_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapEntrySet * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMap* _tmp1_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapEntrySet*) gee_abstract_set_construct (object_type, GEE_MAP_TYPE_ENTRY, (GBoxedCopyFunc) g_object_ref, g_object_unref);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->priv->_map);
	self->priv->_map = _tmp1_;
	return self;
}


static GeeHashMapEntrySet* gee_hash_map_entry_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_entry_set_construct (GEE_HASH_MAP_TYPE_ENTRY_SET, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeIterator* gee_hash_map_entry_set_real_iterator (GeeAbstractCollection* base) {
	GeeHashMapEntrySet * self;
	GeeIterator* result = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMapEntryIterator* _tmp1_ = NULL;
	self = (GeeHashMapEntrySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_hash_map_entry_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp0_);
	result = (GeeIterator*) _tmp1_;
	return result;
}


static gboolean gee_hash_map_entry_set_real_add (GeeAbstractCollection* base, GeeMapEntry* entry) {
	GeeHashMapEntrySet * self;
	gboolean result = FALSE;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	g_assert_not_reached ();
	return result;
}


static void gee_hash_map_entry_set_real_clear (GeeAbstractCollection* base) {
	GeeHashMapEntrySet * self;
	self = (GeeHashMapEntrySet*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_entry_set_real_remove (GeeAbstractCollection* base, GeeMapEntry* entry) {
	GeeHashMapEntrySet * self;
	gboolean result = FALSE;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	g_assert_not_reached ();
	return result;
}


static gboolean gee_hash_map_entry_set_real_contains (GeeAbstractCollection* base, GeeMapEntry* entry) {
	GeeHashMapEntrySet * self;
	gboolean result = FALSE;
	GeeHashMap* _tmp0_ = NULL;
	GeeMapEntry* _tmp1_ = NULL;
	gconstpointer _tmp2_ = NULL;
	gconstpointer _tmp3_ = NULL;
	GeeMapEntry* _tmp4_ = NULL;
	gconstpointer _tmp5_ = NULL;
	gconstpointer _tmp6_ = NULL;
	gboolean _tmp7_ = FALSE;
	self = (GeeHashMapEntrySet*) base;
	g_return_val_if_fail (entry != NULL, FALSE);
	_tmp0_ = self->priv->_map;
	_tmp1_ = entry;
	_tmp2_ = gee_map_entry_get_key (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = entry;
	_tmp5_ = gee_map_entry_get_value (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = gee_abstract_map_has ((GeeAbstractMap*) _tmp0_, _tmp3_, _tmp6_);
	result = _tmp7_;
	return result;
}


static gint gee_hash_map_entry_set_real_get_size (GeeAbstractCollection* base) {
	gint result;
	GeeHashMapEntrySet* self;
	GeeHashMap* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	self = (GeeHashMapEntrySet*) base;
	_tmp0_ = self->priv->_map;
	_tmp1_ = gee_abstract_map_get_size ((GeeMap*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_hash_map_entry_set_real_get_read_only (GeeAbstractCollection* base) {
	gboolean result;
	GeeHashMapEntrySet* self;
	self = (GeeHashMapEntrySet*) base;
	result = TRUE;
	return result;
}


static void gee_hash_map_entry_set_class_init (GeeHashMapEntrySetClass * klass) {
	gee_hash_map_entry_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapEntrySetPrivate));
	((GeeAbstractCollectionClass *) klass)->iterator = gee_hash_map_entry_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = gee_hash_map_entry_set_real_add;
	((GeeAbstractCollectionClass *) klass)->clear = gee_hash_map_entry_set_real_clear;
	((GeeAbstractCollectionClass *) klass)->remove = gee_hash_map_entry_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->contains = gee_hash_map_entry_set_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_hash_map_entry_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_hash_map_entry_set_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_set_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_entry_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_SET_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_entry_set_instance_init (GeeHashMapEntrySet * self) {
	self->priv = GEE_HASH_MAP_ENTRY_SET_GET_PRIVATE (self);
}


static void gee_hash_map_entry_set_finalize (GObject* obj) {
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	_g_object_unref0 (self->priv->_map);
	G_OBJECT_CLASS (gee_hash_map_entry_set_parent_class)->finalize (obj);
}


static GType gee_hash_map_entry_set_get_type (void) {
	static volatile gsize gee_hash_map_entry_set_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_entry_set_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntrySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntrySet), 0, (GInstanceInitFunc) gee_hash_map_entry_set_instance_init, NULL };
		GType gee_hash_map_entry_set_type_id;
		gee_hash_map_entry_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_SET, "GeeHashMapEntrySet", &g_define_type_info, 0);
		g_once_init_leave (&gee_hash_map_entry_set_type_id__volatile, gee_hash_map_entry_set_type_id);
	}
	return gee_hash_map_entry_set_type_id__volatile;
}


static void _vala_gee_hash_map_entry_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_SET_SIZE:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_HASH_MAP_ENTRY_SET_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_entry_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapEntrySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_SET, GeeHashMapEntrySet);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_SET_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_SET_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapNodeIterator * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	GeeHashMap* _tmp1_ = NULL;
	GeeHashMap* _tmp2_ = NULL;
	gint _tmp3_ = 0;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeHashMapNodeIterator*) g_object_new (object_type, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_map);
	self->_map = _tmp1_;
	_tmp2_ = self->_map;
	_tmp3_ = _tmp2_->priv->_stamp;
	self->_stamp = _tmp3_;
	return self;
}


static GeeHashMapNodeIterator* gee_hash_map_node_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapNodeIterator* iter) {
	GeeHashMapNodeIterator * self = NULL;
	GeeHashMapNodeIterator* _tmp0_ = NULL;
	GeeHashMap* _tmp1_ = NULL;
	GeeHashMap* _tmp2_ = NULL;
	GeeHashMapNodeIterator* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	GeeHashMapNodeIterator* _tmp5_ = NULL;
	GeeHashMapNode* _tmp6_ = NULL;
	GeeHashMapNodeIterator* _tmp7_ = NULL;
	GeeHashMapNode* _tmp8_ = NULL;
	GeeHashMapNodeIterator* _tmp9_ = NULL;
	gint _tmp10_ = 0;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeHashMapNodeIterator*) g_object_new (object_type, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = iter;
	_tmp1_ = _tmp0_->_map;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	_g_object_unref0 (self->_map);
	self->_map = _tmp2_;
	_tmp3_ = iter;
	_tmp4_ = _tmp3_->_index;
	self->_index = _tmp4_;
	_tmp5_ = iter;
	_tmp6_ = _tmp5_->_node;
	self->_node = _tmp6_;
	_tmp7_ = iter;
	_tmp8_ = _tmp7_->_next;
	self->_next = _tmp8_;
	_tmp9_ = iter;
	_tmp10_ = _tmp9_->_stamp;
	self->_stamp = _tmp10_;
	return self;
}


static gboolean gee_hash_map_node_iterator_next (GeeHashMapNodeIterator* self) {
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	GeeHashMapNode* _tmp4_ = NULL;
	GeeHashMapNode* _tmp5_ = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = gee_hash_map_node_iterator_has_next (self);
	if (!_tmp3_) {
		result = FALSE;
		return result;
	}
	_tmp4_ = self->_next;
	self->_node = _tmp4_;
	self->_next = NULL;
	_tmp5_ = self->_node;
	result = _tmp5_ != NULL;
	return result;
}


static gboolean gee_hash_map_node_iterator_has_next (GeeHashMapNodeIterator* self) {
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp18_ = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = self->_next;
	if (_tmp3_ == NULL) {
		GeeHashMapNode* _tmp4_ = NULL;
		GeeHashMapNode* _tmp5_ = NULL;
		_tmp4_ = self->_node;
		self->_next = _tmp4_;
		_tmp5_ = self->_next;
		if (_tmp5_ != NULL) {
			GeeHashMapNode* _tmp6_ = NULL;
			GeeHashMapNode* _tmp7_ = NULL;
			_tmp6_ = self->_next;
			_tmp7_ = _tmp6_->next;
			self->_next = _tmp7_;
		}
		while (TRUE) {
			gboolean _tmp8_ = FALSE;
			GeeHashMapNode* _tmp9_ = NULL;
			gint _tmp13_ = 0;
			GeeHashMap* _tmp14_ = NULL;
			GeeHashMapNode** _tmp15_ = NULL;
			gint _tmp15__length1 = 0;
			gint _tmp16_ = 0;
			GeeHashMapNode* _tmp17_ = NULL;
			_tmp9_ = self->_next;
			if (_tmp9_ == NULL) {
				gint _tmp10_ = 0;
				GeeHashMap* _tmp11_ = NULL;
				gint _tmp12_ = 0;
				_tmp10_ = self->_index;
				_tmp11_ = self->_map;
				_tmp12_ = _tmp11_->priv->_array_size;
				_tmp8_ = (_tmp10_ + 1) < _tmp12_;
			} else {
				_tmp8_ = FALSE;
			}
			if (!_tmp8_) {
				break;
			}
			_tmp13_ = self->_index;
			self->_index = _tmp13_ + 1;
			_tmp14_ = self->_map;
			_tmp15_ = _tmp14_->priv->_nodes;
			_tmp15__length1 = _tmp14_->priv->_nodes_length1;
			_tmp16_ = self->_index;
			_tmp17_ = _tmp15_[_tmp16_];
			self->_next = _tmp17_;
		}
	}
	_tmp18_ = self->_next;
	result = _tmp18_ != NULL;
	return result;
}


static gboolean gee_hash_map_node_iterator_get_read_only (GeeHashMapNodeIterator* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_HASH_MAP_NODE_ITERATOR_GET_CLASS (self)->get_read_only (self);
}


static gboolean gee_hash_map_node_iterator_real_get_read_only (GeeHashMapNodeIterator* base) {
	gboolean result;
	GeeHashMapNodeIterator* self;
	self = base;
	result = TRUE;
	return result;
}


static gboolean gee_hash_map_node_iterator_get_valid (GeeHashMapNodeIterator* self) {
	gboolean result;
	GeeHashMapNode* _tmp0_ = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_node;
	result = _tmp0_ != NULL;
	return result;
}


static void gee_hash_map_node_iterator_class_init (GeeHashMapNodeIteratorClass * klass) {
	gee_hash_map_node_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapNodeIteratorPrivate));
	GEE_HASH_MAP_NODE_ITERATOR_CLASS (klass)->get_read_only = gee_hash_map_node_iterator_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_node_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_node_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_node_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_NODE_ITERATOR_VALID, g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_node_iterator_instance_init (GeeHashMapNodeIterator * self) {
	self->priv = GEE_HASH_MAP_NODE_ITERATOR_GET_PRIVATE (self);
	self->_index = -1;
}


static void gee_hash_map_node_iterator_finalize (GObject* obj) {
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	_g_object_unref0 (self->_map);
	G_OBJECT_CLASS (gee_hash_map_node_iterator_parent_class)->finalize (obj);
}


static GType gee_hash_map_node_iterator_get_type (void) {
	static volatile gsize gee_hash_map_node_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_node_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapNodeIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_node_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapNodeIterator), 0, (GInstanceInitFunc) gee_hash_map_node_iterator_instance_init, NULL };
		GType gee_hash_map_node_iterator_type_id;
		gee_hash_map_node_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapNodeIterator", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&gee_hash_map_node_iterator_type_id__volatile, gee_hash_map_node_iterator_type_id);
	}
	return gee_hash_map_node_iterator_type_id__volatile;
}


static void _vala_gee_hash_map_node_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	switch (property_id) {
		case GEE_HASH_MAP_NODE_ITERATOR_READ_ONLY:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_read_only (self));
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_VALID:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_valid (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_node_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapNodeIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_NODE_ITERATOR, GeeHashMapNodeIterator);
	switch (property_id) {
		case GEE_HASH_MAP_NODE_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_NODE_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapKeyIterator * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	_tmp0_ = map;
	self = (GeeHashMapKeyIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_key_iterator_construct (GEE_HASH_MAP_TYPE_KEY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapKeyIterator* iter) {
	GeeHashMapKeyIterator * self = NULL;
	GeeHashMapKeyIterator* _tmp0_ = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	_tmp0_ = iter;
	self = (GeeHashMapKeyIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, (GeeHashMapNodeIterator*) _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapKeyIterator* iter) {
	return gee_hash_map_key_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_KEY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}


static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base) {
	GeeHashMapKeyIterator * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp4_ = NULL;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	self = (GeeHashMapKeyIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp5_ = _tmp4_->key;
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	result = _tmp6_;
	return result;
}


static void gee_hash_map_key_iterator_real_remove (GeeIterator* base) {
	GeeHashMapKeyIterator * self;
	self = (GeeHashMapKeyIterator*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_key_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeHashMapKeyIterator * self;
	gboolean result = FALSE;
	GeeHashMapNode* _tmp0_ = NULL;
	self = (GeeHashMapKeyIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeForallFunc _tmp1_ = NULL;
		void* _tmp1__target = NULL;
		GeeHashMapNode* _tmp2_ = NULL;
		gconstpointer _tmp3_ = NULL;
		gpointer _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		GeeHashMapNode* _tmp6_ = NULL;
		_tmp1_ = f;
		_tmp1__target = f_target;
		_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp3_ = _tmp2_->key;
		_tmp4_ = ((_tmp3_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
		_tmp5_ = _tmp1_ (_tmp4_, _tmp1__target);
		if (!_tmp5_) {
			result = FALSE;
			return result;
		}
		_tmp6_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp6_ == NULL) {
			GeeHashMapNode* _tmp7_ = NULL;
			GeeHashMapNode* _tmp8_ = NULL;
			_tmp7_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp8_ = _tmp7_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp8_;
		}
	}
	while (TRUE) {
		gint _tmp18_ = 0;
		GeeHashMap* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		while (TRUE) {
			GeeHashMapNode* _tmp9_ = NULL;
			GeeHashMapNode* _tmp10_ = NULL;
			GeeForallFunc _tmp11_ = NULL;
			void* _tmp11__target = NULL;
			GeeHashMapNode* _tmp12_ = NULL;
			gconstpointer _tmp13_ = NULL;
			gpointer _tmp14_ = NULL;
			gboolean _tmp15_ = FALSE;
			GeeHashMapNode* _tmp16_ = NULL;
			GeeHashMapNode* _tmp17_ = NULL;
			_tmp9_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp9_ != NULL)) {
				break;
			}
			_tmp10_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp10_;
			_tmp11_ = f;
			_tmp11__target = f_target;
			_tmp12_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp13_ = _tmp12_->key;
			_tmp14_ = ((_tmp13_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp13_) : ((gpointer) _tmp13_);
			_tmp15_ = _tmp11_ (_tmp14_, _tmp11__target);
			if (!_tmp15_) {
				result = FALSE;
				return result;
			}
			_tmp16_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp17_ = _tmp16_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp17_;
		}
		_tmp18_ = ((GeeHashMapNodeIterator*) self)->_index;
		_tmp19_ = ((GeeHashMapNodeIterator*) self)->_map;
		_tmp20_ = _tmp19_->priv->_array_size;
		if ((_tmp18_ + 1) < _tmp20_) {
			GeeHashMap* _tmp21_ = NULL;
			GeeHashMapNode** _tmp22_ = NULL;
			gint _tmp22__length1 = 0;
			gint _tmp23_ = 0;
			gint _tmp24_ = 0;
			GeeHashMapNode* _tmp25_ = NULL;
			_tmp21_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp22_ = _tmp21_->priv->_nodes;
			_tmp22__length1 = _tmp21_->priv->_nodes_length1;
			_tmp23_ = ((GeeHashMapNodeIterator*) self)->_index;
			((GeeHashMapNodeIterator*) self)->_index = _tmp23_ + 1;
			_tmp24_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp25_ = _tmp22_[_tmp24_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp25_;
		} else {
			result = TRUE;
			return result;
		}
	}
}


static GeeIterator** gee_hash_map_key_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1) {
	GeeHashMapKeyIterator * self;
	GeeIterator** result = NULL;
	guint _tmp0_ = 0U;
	self = (GeeHashMapKeyIterator*) base;
	_tmp0_ = forks;
	if (_tmp0_ == ((guint) 0)) {
		GeeIterator** _tmp1_ = NULL;
		GeeIterator** _tmp2_ = NULL;
		gint _tmp2__length1 = 0;
		_tmp1_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp2_ = _tmp1_;
		_tmp2__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp2__length1;
		}
		result = _tmp2_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		guint _tmp3_ = 0U;
		GeeIterator** _tmp4_ = NULL;
		gint _result__length1 = 0;
		gint __result__size_ = 0;
		GeeIterator** _tmp5_ = NULL;
		gint _tmp5__length1 = 0;
		GeeIterator* _tmp6_ = NULL;
		GeeIterator* _tmp7_ = NULL;
		GeeIterator** _tmp16_ = NULL;
		gint _tmp16__length1 = 0;
		_tmp3_ = forks;
		_tmp4_ = g_new0 (GeeIterator*, _tmp3_ + 1);
		_result_ = _tmp4_;
		_result__length1 = _tmp3_;
		__result__size_ = _result__length1;
		_tmp5_ = _result_;
		_tmp5__length1 = _result__length1;
		_tmp6_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp5_[0]);
		_tmp5_[0] = _tmp6_;
		_tmp7_ = _tmp5_[0];
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp8_ = FALSE;
				_tmp8_ = TRUE;
				while (TRUE) {
					guint _tmp10_ = 0U;
					guint _tmp11_ = 0U;
					GeeIterator** _tmp12_ = NULL;
					gint _tmp12__length1 = 0;
					guint _tmp13_ = 0U;
					GeeHashMapKeyIterator* _tmp14_ = NULL;
					GeeIterator* _tmp15_ = NULL;
					if (!_tmp8_) {
						guint _tmp9_ = 0U;
						_tmp9_ = i;
						i = _tmp9_ + 1;
					}
					_tmp8_ = FALSE;
					_tmp10_ = i;
					_tmp11_ = forks;
					if (!(_tmp10_ < _tmp11_)) {
						break;
					}
					_tmp12_ = _result_;
					_tmp12__length1 = _result__length1;
					_tmp13_ = i;
					_tmp14_ = gee_hash_map_key_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp12_[_tmp13_]);
					_tmp12_[_tmp13_] = (GeeIterator*) _tmp14_;
					_tmp15_ = _tmp12_[_tmp13_];
				}
			}
		}
		_tmp16_ = _result_;
		_tmp16__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp16__length1;
		}
		result = _tmp16_;
		return result;
	}
}


static void gee_hash_map_key_iterator_class_init (GeeHashMapKeyIteratorClass * klass) {
	gee_hash_map_key_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapKeyIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_key_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_key_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static GType gee_hash_map_key_iterator_gee_traversable_get_g_type (GeeHashMapKeyIterator* self) {
	return self->priv->k_type;
}


static GBoxedCopyFunc gee_hash_map_key_iterator_gee_traversable_get_g_dup_func (GeeHashMapKeyIterator* self) {
	return self->priv->k_dup_func;
}


static GDestroyNotify gee_hash_map_key_iterator_gee_traversable_get_g_destroy_func (GeeHashMapKeyIterator* self) {
	return self->priv->k_destroy_func;
}


static void gee_hash_map_key_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_hash_map_key_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_hash_map_key_iterator_real_foreach;
	iface->tee = (GeeIterator** (*)(GeeTraversable*, guint, int*)) gee_hash_map_key_iterator_real_tee;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_hash_map_key_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_hash_map_key_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_hash_map_key_iterator_gee_traversable_get_g_destroy_func;
}


static void gee_hash_map_key_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_hash_map_key_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*)(GeeIterator*)) gee_hash_map_key_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_hash_map_key_iterator_real_remove;
	iface->next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}


static void gee_hash_map_key_iterator_instance_init (GeeHashMapKeyIterator * self) {
	self->priv = GEE_HASH_MAP_KEY_ITERATOR_GET_PRIVATE (self);
}


static GType gee_hash_map_key_iterator_get_type (void) {
	static volatile gsize gee_hash_map_key_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_key_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeyIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeyIterator), 0, (GInstanceInitFunc) gee_hash_map_key_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_key_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_key_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_hash_map_key_iterator_type_id;
		gee_hash_map_key_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapKeyIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_key_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_hash_map_key_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_once_init_leave (&gee_hash_map_key_iterator_type_id__volatile, gee_hash_map_key_iterator_type_id);
	}
	return gee_hash_map_key_iterator_type_id__volatile;
}


static void _vala_gee_hash_map_key_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapKeyIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_key_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapKeyIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapMapIterator* gee_hash_map_map_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapMapIterator * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	_tmp0_ = map;
	self = (GeeHashMapMapIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapMapIterator* gee_hash_map_map_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_map_iterator_construct (GEE_HASH_MAP_TYPE_MAP_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static gpointer gee_hash_map_map_iterator_real_get_key (GeeMapIterator* base) {
	GeeHashMapMapIterator * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp4_ = NULL;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp5_ = _tmp4_->key;
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->k_dup_func != NULL)) ? self->priv->k_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	result = _tmp6_;
	return result;
}


static void gee_hash_map_map_iterator_real_unset (GeeMapIterator* base) {
	GeeHashMapMapIterator * self;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMap* _tmp4_ = NULL;
	GeeHashMapNode* _tmp5_ = NULL;
	gconstpointer _tmp6_ = NULL;
	GeeHashMap* _tmp7_ = NULL;
	gint _tmp8_ = 0;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	gee_map_iterator_has_next ((GeeMapIterator*) self);
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp5_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp6_ = _tmp5_->key;
	gee_hash_map_unset_helper (_tmp4_, _tmp6_, NULL);
	((GeeHashMapNodeIterator*) self)->_node = NULL;
	_tmp7_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp8_ = _tmp7_->priv->_stamp;
	((GeeHashMapNodeIterator*) self)->_stamp = _tmp8_;
}


static gpointer gee_hash_map_map_iterator_real_get_value (GeeMapIterator* base) {
	GeeHashMapMapIterator * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp4_ = NULL;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp5_ = _tmp4_->value;
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	result = _tmp6_;
	return result;
}


static void gee_hash_map_map_iterator_real_set_value (GeeMapIterator* base, gconstpointer value) {
	GeeHashMapMapIterator * self;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMap* _tmp4_ = NULL;
	GeeHashMapNode* _tmp5_ = NULL;
	gconstpointer _tmp6_ = NULL;
	gconstpointer _tmp7_ = NULL;
	GeeHashMap* _tmp8_ = NULL;
	gint _tmp9_ = 0;
	self = (GeeHashMapMapIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp5_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp6_ = _tmp5_->key;
	_tmp7_ = value;
	gee_abstract_map_set ((GeeAbstractMap*) _tmp4_, _tmp6_, _tmp7_);
	_tmp8_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp9_ = _tmp8_->priv->_stamp;
	((GeeHashMapNodeIterator*) self)->_stamp = _tmp9_;
}


static gboolean gee_hash_map_map_iterator_real_get_mutable (GeeMapIterator* base) {
	gboolean result;
	GeeHashMapMapIterator* self;
	self = (GeeHashMapMapIterator*) base;
	result = TRUE;
	return result;
}


static gboolean gee_hash_map_map_iterator_real_get_read_only (GeeHashMapNodeIterator* base) {
	gboolean result;
	GeeHashMapMapIterator* self;
	self = (GeeHashMapMapIterator*) base;
	result = FALSE;
	return result;
}


static void gee_hash_map_map_iterator_class_init (GeeHashMapMapIteratorClass * klass) {
	gee_hash_map_map_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapMapIteratorPrivate));
	GEE_HASH_MAP_NODE_ITERATOR_CLASS (klass)->get_read_only = gee_hash_map_map_iterator_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_map_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_map_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_MUTABLE, g_param_spec_boolean ("mutable", "mutable", "mutable", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_hash_map_map_iterator_gee_map_iterator_get_k_type (GeeHashMapMapIterator* self) {
	return self->priv->k_type;
}


static GBoxedCopyFunc gee_hash_map_map_iterator_gee_map_iterator_get_k_dup_func (GeeHashMapMapIterator* self) {
	return self->priv->k_dup_func;
}


static GDestroyNotify gee_hash_map_map_iterator_gee_map_iterator_get_k_destroy_func (GeeHashMapMapIterator* self) {
	return self->priv->k_destroy_func;
}


static GType gee_hash_map_map_iterator_gee_map_iterator_get_v_type (GeeHashMapMapIterator* self) {
	return self->priv->v_type;
}


static GBoxedCopyFunc gee_hash_map_map_iterator_gee_map_iterator_get_v_dup_func (GeeHashMapMapIterator* self) {
	return self->priv->v_dup_func;
}


static GDestroyNotify gee_hash_map_map_iterator_gee_map_iterator_get_v_destroy_func (GeeHashMapMapIterator* self) {
	return self->priv->v_destroy_func;
}


static void gee_hash_map_map_iterator_gee_map_iterator_interface_init (GeeMapIteratorIface * iface) {
	gee_hash_map_map_iterator_gee_map_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_key = (gpointer (*)(GeeMapIterator*)) gee_hash_map_map_iterator_real_get_key;
	iface->unset = (void (*)(GeeMapIterator*)) gee_hash_map_map_iterator_real_unset;
	iface->get_value = (gpointer (*)(GeeMapIterator*)) gee_hash_map_map_iterator_real_get_value;
	iface->set_value = (void (*)(GeeMapIterator*, gconstpointer)) gee_hash_map_map_iterator_real_set_value;
	iface->get_k_type = (GType(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_k_destroy_func;
	iface->get_v_type = (GType(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify(*)(GeeMapIterator*)) gee_hash_map_map_iterator_gee_map_iterator_get_v_destroy_func;
	iface->next = (gboolean (*)(GeeMapIterator*)) gee_hash_map_node_iterator_next;
	iface->has_next = (gboolean (*)(GeeMapIterator*)) gee_hash_map_node_iterator_has_next;
	iface->get_mutable = gee_hash_map_map_iterator_real_get_mutable;
	iface->get_read_only = gee_hash_map_map_iterator_real_get_read_only;
	iface->get_valid = (gboolean (*) (GeeMapIterator *)) gee_hash_map_node_iterator_get_valid;
}


static void gee_hash_map_map_iterator_instance_init (GeeHashMapMapIterator * self) {
	self->priv = GEE_HASH_MAP_MAP_ITERATOR_GET_PRIVATE (self);
}


static GType gee_hash_map_map_iterator_get_type (void) {
	static volatile gsize gee_hash_map_map_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_map_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapMapIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_map_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapMapIterator), 0, (GInstanceInitFunc) gee_hash_map_map_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_map_iterator_info = { (GInterfaceInitFunc) gee_hash_map_map_iterator_gee_map_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_hash_map_map_iterator_type_id;
		gee_hash_map_map_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapMapIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_map_iterator_type_id, GEE_TYPE_MAP_ITERATOR, &gee_map_iterator_info);
		g_once_init_leave (&gee_hash_map_map_iterator_type_id__volatile, gee_hash_map_map_iterator_type_id);
	}
	return gee_hash_map_map_iterator_type_id__volatile;
}


static void _vala_gee_hash_map_map_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator);
	switch (property_id) {
		case GEE_HASH_MAP_MAP_ITERATOR_MUTABLE:
		g_value_set_boolean (value, gee_map_iterator_get_mutable ((GeeMapIterator*) self));
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_READ_ONLY:
		g_value_set_boolean (value, gee_hash_map_node_iterator_get_read_only ((GeeHashMapNodeIterator*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_map_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_MAP_ITERATOR, GeeHashMapMapIterator);
	switch (property_id) {
		case GEE_HASH_MAP_MAP_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_MAP_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapValueIterator * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	_tmp0_ = map;
	self = (GeeHashMapValueIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_value_iterator_construct (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapValueIterator* iter) {
	GeeHashMapValueIterator * self = NULL;
	GeeHashMapValueIterator* _tmp0_ = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	_tmp0_ = iter;
	self = (GeeHashMapValueIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, (GeeHashMapNodeIterator*) _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapValueIterator* iter) {
	return gee_hash_map_value_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}


static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base) {
	GeeHashMapValueIterator * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp4_ = NULL;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	self = (GeeHashMapValueIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp5_ = _tmp4_->value;
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	result = _tmp6_;
	return result;
}


static void gee_hash_map_value_iterator_real_remove (GeeIterator* base) {
	GeeHashMapValueIterator * self;
	self = (GeeHashMapValueIterator*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_value_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeHashMapValueIterator * self;
	gboolean result = FALSE;
	GeeHashMapNode* _tmp0_ = NULL;
	self = (GeeHashMapValueIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeForallFunc _tmp1_ = NULL;
		void* _tmp1__target = NULL;
		GeeHashMapNode* _tmp2_ = NULL;
		gconstpointer _tmp3_ = NULL;
		gpointer _tmp4_ = NULL;
		gboolean _tmp5_ = FALSE;
		GeeHashMapNode* _tmp6_ = NULL;
		_tmp1_ = f;
		_tmp1__target = f_target;
		_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp3_ = _tmp2_->value;
		_tmp4_ = ((_tmp3_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp3_) : ((gpointer) _tmp3_);
		_tmp5_ = _tmp1_ (_tmp4_, _tmp1__target);
		if (!_tmp5_) {
			result = FALSE;
			return result;
		}
		_tmp6_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp6_ == NULL) {
			GeeHashMapNode* _tmp7_ = NULL;
			GeeHashMapNode* _tmp8_ = NULL;
			_tmp7_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp8_ = _tmp7_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp8_;
		}
	}
	while (TRUE) {
		gint _tmp18_ = 0;
		GeeHashMap* _tmp19_ = NULL;
		gint _tmp20_ = 0;
		while (TRUE) {
			GeeHashMapNode* _tmp9_ = NULL;
			GeeHashMapNode* _tmp10_ = NULL;
			GeeForallFunc _tmp11_ = NULL;
			void* _tmp11__target = NULL;
			GeeHashMapNode* _tmp12_ = NULL;
			gconstpointer _tmp13_ = NULL;
			gpointer _tmp14_ = NULL;
			gboolean _tmp15_ = FALSE;
			GeeHashMapNode* _tmp16_ = NULL;
			GeeHashMapNode* _tmp17_ = NULL;
			_tmp9_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp9_ != NULL)) {
				break;
			}
			_tmp10_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp10_;
			_tmp11_ = f;
			_tmp11__target = f_target;
			_tmp12_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp13_ = _tmp12_->value;
			_tmp14_ = ((_tmp13_ != NULL) && (self->priv->v_dup_func != NULL)) ? self->priv->v_dup_func ((gpointer) _tmp13_) : ((gpointer) _tmp13_);
			_tmp15_ = _tmp11_ (_tmp14_, _tmp11__target);
			if (!_tmp15_) {
				result = FALSE;
				return result;
			}
			_tmp16_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp17_ = _tmp16_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp17_;
		}
		_tmp18_ = ((GeeHashMapNodeIterator*) self)->_index;
		_tmp19_ = ((GeeHashMapNodeIterator*) self)->_map;
		_tmp20_ = _tmp19_->priv->_array_size;
		if ((_tmp18_ + 1) < _tmp20_) {
			GeeHashMap* _tmp21_ = NULL;
			GeeHashMapNode** _tmp22_ = NULL;
			gint _tmp22__length1 = 0;
			gint _tmp23_ = 0;
			gint _tmp24_ = 0;
			GeeHashMapNode* _tmp25_ = NULL;
			_tmp21_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp22_ = _tmp21_->priv->_nodes;
			_tmp22__length1 = _tmp21_->priv->_nodes_length1;
			_tmp23_ = ((GeeHashMapNodeIterator*) self)->_index;
			((GeeHashMapNodeIterator*) self)->_index = _tmp23_ + 1;
			_tmp24_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp25_ = _tmp22_[_tmp24_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp25_;
		} else {
			result = TRUE;
			return result;
		}
	}
}


static GeeIterator** gee_hash_map_value_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1) {
	GeeHashMapValueIterator * self;
	GeeIterator** result = NULL;
	guint _tmp0_ = 0U;
	self = (GeeHashMapValueIterator*) base;
	_tmp0_ = forks;
	if (_tmp0_ == ((guint) 0)) {
		GeeIterator** _tmp1_ = NULL;
		GeeIterator** _tmp2_ = NULL;
		gint _tmp2__length1 = 0;
		_tmp1_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp2_ = _tmp1_;
		_tmp2__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp2__length1;
		}
		result = _tmp2_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		guint _tmp3_ = 0U;
		GeeIterator** _tmp4_ = NULL;
		gint _result__length1 = 0;
		gint __result__size_ = 0;
		GeeIterator** _tmp5_ = NULL;
		gint _tmp5__length1 = 0;
		GeeIterator* _tmp6_ = NULL;
		GeeIterator* _tmp7_ = NULL;
		GeeIterator** _tmp16_ = NULL;
		gint _tmp16__length1 = 0;
		_tmp3_ = forks;
		_tmp4_ = g_new0 (GeeIterator*, _tmp3_ + 1);
		_result_ = _tmp4_;
		_result__length1 = _tmp3_;
		__result__size_ = _result__length1;
		_tmp5_ = _result_;
		_tmp5__length1 = _result__length1;
		_tmp6_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp5_[0]);
		_tmp5_[0] = _tmp6_;
		_tmp7_ = _tmp5_[0];
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp8_ = FALSE;
				_tmp8_ = TRUE;
				while (TRUE) {
					guint _tmp10_ = 0U;
					guint _tmp11_ = 0U;
					GeeIterator** _tmp12_ = NULL;
					gint _tmp12__length1 = 0;
					guint _tmp13_ = 0U;
					GeeHashMapValueIterator* _tmp14_ = NULL;
					GeeIterator* _tmp15_ = NULL;
					if (!_tmp8_) {
						guint _tmp9_ = 0U;
						_tmp9_ = i;
						i = _tmp9_ + 1;
					}
					_tmp8_ = FALSE;
					_tmp10_ = i;
					_tmp11_ = forks;
					if (!(_tmp10_ < _tmp11_)) {
						break;
					}
					_tmp12_ = _result_;
					_tmp12__length1 = _result__length1;
					_tmp13_ = i;
					_tmp14_ = gee_hash_map_value_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp12_[_tmp13_]);
					_tmp12_[_tmp13_] = (GeeIterator*) _tmp14_;
					_tmp15_ = _tmp12_[_tmp13_];
				}
			}
		}
		_tmp16_ = _result_;
		_tmp16__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp16__length1;
		}
		result = _tmp16_;
		return result;
	}
}


static void gee_hash_map_value_iterator_class_init (GeeHashMapValueIteratorClass * klass) {
	gee_hash_map_value_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapValueIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_value_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_value_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static GType gee_hash_map_value_iterator_gee_traversable_get_g_type (GeeHashMapValueIterator* self) {
	return self->priv->v_type;
}


static GBoxedCopyFunc gee_hash_map_value_iterator_gee_traversable_get_g_dup_func (GeeHashMapValueIterator* self) {
	return self->priv->v_dup_func;
}


static GDestroyNotify gee_hash_map_value_iterator_gee_traversable_get_g_destroy_func (GeeHashMapValueIterator* self) {
	return self->priv->v_destroy_func;
}


static void gee_hash_map_value_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_hash_map_value_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_hash_map_value_iterator_real_foreach;
	iface->tee = (GeeIterator** (*)(GeeTraversable*, guint, int*)) gee_hash_map_value_iterator_real_tee;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_hash_map_value_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_hash_map_value_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_hash_map_value_iterator_gee_traversable_get_g_destroy_func;
}


static void gee_hash_map_value_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_hash_map_value_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*)(GeeIterator*)) gee_hash_map_value_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_hash_map_value_iterator_real_remove;
	iface->next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}


static void gee_hash_map_value_iterator_instance_init (GeeHashMapValueIterator * self) {
	self->priv = GEE_HASH_MAP_VALUE_ITERATOR_GET_PRIVATE (self);
}


static GType gee_hash_map_value_iterator_get_type (void) {
	static volatile gsize gee_hash_map_value_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_value_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueIterator), 0, (GInstanceInitFunc) gee_hash_map_value_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_value_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_value_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_hash_map_value_iterator_type_id;
		gee_hash_map_value_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapValueIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_value_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_hash_map_value_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_once_init_leave (&gee_hash_map_value_iterator_type_id__volatile, gee_hash_map_value_iterator_type_id);
	}
	return gee_hash_map_value_iterator_type_id__volatile;
}


static void _vala_gee_hash_map_value_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapValueIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_value_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapValueIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GeeHashMapEntryIterator * self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	_tmp0_ = map;
	self = (GeeHashMapEntryIterator*) gee_hash_map_node_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_entry_iterator_construct (GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_construct_from_iterator (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapEntryIterator* iter) {
	GeeHashMapEntryIterator * self = NULL;
	GeeHashMapEntryIterator* _tmp0_ = NULL;
	g_return_val_if_fail (iter != NULL, NULL);
	_tmp0_ = iter;
	self = (GeeHashMapEntryIterator*) gee_hash_map_node_iterator_construct_from_iterator (object_type, k_type, (GBoxedCopyFunc) k_dup_func, k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, v_destroy_func, (GeeHashMapNodeIterator*) _tmp0_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


static GeeHashMapEntryIterator* gee_hash_map_entry_iterator_new_from_iterator (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMapEntryIterator* iter) {
	return gee_hash_map_entry_iterator_construct_from_iterator (GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iter);
}


static GeeMapEntry* gee_hash_map_entry_iterator_real_get (GeeIterator* base) {
	GeeHashMapEntryIterator * self;
	GeeMapEntry* result = NULL;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeHashMapNode* _tmp3_ = NULL;
	GeeHashMapNode* _tmp4_ = NULL;
	GeeMapEntry* _tmp5_ = NULL;
	self = (GeeHashMapEntryIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_stamp;
	_tmp1_ = ((GeeHashMapNodeIterator*) self)->_map;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _map._stamp");
	_tmp3_ = ((GeeHashMapNodeIterator*) self)->_node;
	_vala_assert (_tmp3_ != NULL, "_node != null");
	_tmp4_ = ((GeeHashMapNodeIterator*) self)->_node;
	_tmp5_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp4_);
	result = _tmp5_;
	return result;
}


static void gee_hash_map_entry_iterator_real_remove (GeeIterator* base) {
	GeeHashMapEntryIterator * self;
	self = (GeeHashMapEntryIterator*) base;
	g_assert_not_reached ();
}


static gboolean gee_hash_map_entry_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeHashMapEntryIterator * self;
	gboolean result = FALSE;
	GeeHashMapNode* _tmp0_ = NULL;
	self = (GeeHashMapEntryIterator*) base;
	_tmp0_ = ((GeeHashMapNodeIterator*) self)->_node;
	if (_tmp0_ != NULL) {
		GeeForallFunc _tmp1_ = NULL;
		void* _tmp1__target = NULL;
		GeeHashMapNode* _tmp2_ = NULL;
		GeeMapEntry* _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		GeeHashMapNode* _tmp5_ = NULL;
		_tmp1_ = f;
		_tmp1__target = f_target;
		_tmp2_ = ((GeeHashMapNodeIterator*) self)->_node;
		_tmp3_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp2_);
		_tmp4_ = _tmp1_ (_tmp3_, _tmp1__target);
		if (!_tmp4_) {
			result = FALSE;
			return result;
		}
		_tmp5_ = ((GeeHashMapNodeIterator*) self)->_next;
		if (_tmp5_ == NULL) {
			GeeHashMapNode* _tmp6_ = NULL;
			GeeHashMapNode* _tmp7_ = NULL;
			_tmp6_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp7_ = _tmp6_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp7_;
		}
	}
	while (TRUE) {
		gint _tmp16_ = 0;
		GeeHashMap* _tmp17_ = NULL;
		gint _tmp18_ = 0;
		while (TRUE) {
			GeeHashMapNode* _tmp8_ = NULL;
			GeeHashMapNode* _tmp9_ = NULL;
			GeeForallFunc _tmp10_ = NULL;
			void* _tmp10__target = NULL;
			GeeHashMapNode* _tmp11_ = NULL;
			GeeMapEntry* _tmp12_ = NULL;
			gboolean _tmp13_ = FALSE;
			GeeHashMapNode* _tmp14_ = NULL;
			GeeHashMapNode* _tmp15_ = NULL;
			_tmp8_ = ((GeeHashMapNodeIterator*) self)->_next;
			if (!(_tmp8_ != NULL)) {
				break;
			}
			_tmp9_ = ((GeeHashMapNodeIterator*) self)->_next;
			((GeeHashMapNodeIterator*) self)->_node = _tmp9_;
			_tmp10_ = f;
			_tmp10__target = f_target;
			_tmp11_ = ((GeeHashMapNodeIterator*) self)->_node;
			_tmp12_ = gee_hash_map_entry_entry_for (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, _tmp11_);
			_tmp13_ = _tmp10_ (_tmp12_, _tmp10__target);
			if (!_tmp13_) {
				result = FALSE;
				return result;
			}
			_tmp14_ = ((GeeHashMapNodeIterator*) self)->_next;
			_tmp15_ = _tmp14_->next;
			((GeeHashMapNodeIterator*) self)->_next = _tmp15_;
		}
		_tmp16_ = ((GeeHashMapNodeIterator*) self)->_index;
		_tmp17_ = ((GeeHashMapNodeIterator*) self)->_map;
		_tmp18_ = _tmp17_->priv->_array_size;
		if ((_tmp16_ + 1) < _tmp18_) {
			GeeHashMap* _tmp19_ = NULL;
			GeeHashMapNode** _tmp20_ = NULL;
			gint _tmp20__length1 = 0;
			gint _tmp21_ = 0;
			gint _tmp22_ = 0;
			GeeHashMapNode* _tmp23_ = NULL;
			_tmp19_ = ((GeeHashMapNodeIterator*) self)->_map;
			_tmp20_ = _tmp19_->priv->_nodes;
			_tmp20__length1 = _tmp19_->priv->_nodes_length1;
			_tmp21_ = ((GeeHashMapNodeIterator*) self)->_index;
			((GeeHashMapNodeIterator*) self)->_index = _tmp21_ + 1;
			_tmp22_ = ((GeeHashMapNodeIterator*) self)->_index;
			_tmp23_ = _tmp20_[_tmp22_];
			((GeeHashMapNodeIterator*) self)->_next = _tmp23_;
		} else {
			result = TRUE;
			return result;
		}
	}
}


static GeeIterator** gee_hash_map_entry_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1) {
	GeeHashMapEntryIterator * self;
	GeeIterator** result = NULL;
	guint _tmp0_ = 0U;
	self = (GeeHashMapEntryIterator*) base;
	_tmp0_ = forks;
	if (_tmp0_ == ((guint) 0)) {
		GeeIterator** _tmp1_ = NULL;
		GeeIterator** _tmp2_ = NULL;
		gint _tmp2__length1 = 0;
		_tmp1_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp2_ = _tmp1_;
		_tmp2__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp2__length1;
		}
		result = _tmp2_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		guint _tmp3_ = 0U;
		GeeIterator** _tmp4_ = NULL;
		gint _result__length1 = 0;
		gint __result__size_ = 0;
		GeeIterator** _tmp5_ = NULL;
		gint _tmp5__length1 = 0;
		GeeIterator* _tmp6_ = NULL;
		GeeIterator* _tmp7_ = NULL;
		GeeIterator** _tmp16_ = NULL;
		gint _tmp16__length1 = 0;
		_tmp3_ = forks;
		_tmp4_ = g_new0 (GeeIterator*, _tmp3_ + 1);
		_result_ = _tmp4_;
		_result__length1 = _tmp3_;
		__result__size_ = _result__length1;
		_tmp5_ = _result_;
		_tmp5__length1 = _result__length1;
		_tmp6_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp5_[0]);
		_tmp5_[0] = _tmp6_;
		_tmp7_ = _tmp5_[0];
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp8_ = FALSE;
				_tmp8_ = TRUE;
				while (TRUE) {
					guint _tmp10_ = 0U;
					guint _tmp11_ = 0U;
					GeeIterator** _tmp12_ = NULL;
					gint _tmp12__length1 = 0;
					guint _tmp13_ = 0U;
					GeeHashMapEntryIterator* _tmp14_ = NULL;
					GeeIterator* _tmp15_ = NULL;
					if (!_tmp8_) {
						guint _tmp9_ = 0U;
						_tmp9_ = i;
						i = _tmp9_ + 1;
					}
					_tmp8_ = FALSE;
					_tmp10_ = i;
					_tmp11_ = forks;
					if (!(_tmp10_ < _tmp11_)) {
						break;
					}
					_tmp12_ = _result_;
					_tmp12__length1 = _result__length1;
					_tmp13_ = i;
					_tmp14_ = gee_hash_map_entry_iterator_new_from_iterator (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, self->priv->v_destroy_func, self);
					_g_object_unref0 (_tmp12_[_tmp13_]);
					_tmp12_[_tmp13_] = (GeeIterator*) _tmp14_;
					_tmp15_ = _tmp12_[_tmp13_];
				}
			}
		}
		_tmp16_ = _result_;
		_tmp16__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp16__length1;
		}
		result = _tmp16_;
		return result;
	}
}


static void gee_hash_map_entry_iterator_class_init (GeeHashMapEntryIteratorClass * klass) {
	gee_hash_map_entry_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapEntryIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_entry_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_entry_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static GType gee_hash_map_entry_iterator_gee_traversable_get_g_type (GeeHashMapEntryIterator* self) {
	return GEE_MAP_TYPE_ENTRY;
}


static GBoxedCopyFunc gee_hash_map_entry_iterator_gee_traversable_get_g_dup_func (GeeHashMapEntryIterator* self) {
	return g_object_ref;
}


static GDestroyNotify gee_hash_map_entry_iterator_gee_traversable_get_g_destroy_func (GeeHashMapEntryIterator* self) {
	return g_object_unref;
}


static void gee_hash_map_entry_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_hash_map_entry_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_hash_map_entry_iterator_real_foreach;
	iface->tee = (GeeIterator** (*)(GeeTraversable*, guint, int*)) gee_hash_map_entry_iterator_real_tee;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_hash_map_entry_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_hash_map_entry_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_hash_map_entry_iterator_gee_traversable_get_g_destroy_func;
}


static void gee_hash_map_entry_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_hash_map_entry_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*)(GeeIterator*)) gee_hash_map_entry_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_hash_map_entry_iterator_real_remove;
	iface->next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_hash_map_node_iterator_has_next;
	iface->get_valid = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_valid;
	iface->get_read_only = (gboolean (*) (GeeIterator *)) gee_hash_map_node_iterator_get_read_only;
}


static void gee_hash_map_entry_iterator_instance_init (GeeHashMapEntryIterator * self) {
	self->priv = GEE_HASH_MAP_ENTRY_ITERATOR_GET_PRIVATE (self);
}


static GType gee_hash_map_entry_iterator_get_type (void) {
	static volatile gsize gee_hash_map_entry_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_entry_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapEntryIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_entry_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapEntryIterator), 0, (GInstanceInitFunc) gee_hash_map_entry_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_hash_map_entry_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_entry_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_hash_map_entry_iterator_type_id;
		gee_hash_map_entry_iterator_type_id = g_type_register_static (GEE_HASH_MAP_TYPE_NODE_ITERATOR, "GeeHashMapEntryIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_entry_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_hash_map_entry_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_once_init_leave (&gee_hash_map_entry_iterator_type_id__volatile, gee_hash_map_entry_iterator_type_id);
	}
	return gee_hash_map_entry_iterator_type_id__volatile;
}


static void _vala_gee_hash_map_entry_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapEntryIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_entry_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapEntryIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_HASH_MAP_TYPE_ENTRY_ITERATOR, GeeHashMapEntryIterator);
	switch (property_id) {
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_ENTRY_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_class_init (GeeHashMapClass * klass) {
	gee_hash_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapPrivate));
	((GeeAbstractMapClass *) klass)->has_key = gee_hash_map_real_has_key;
	((GeeAbstractMapClass *) klass)->has = gee_hash_map_real_has;
	((GeeAbstractMapClass *) klass)->get = gee_hash_map_real_get;
	((GeeAbstractMapClass *) klass)->set = gee_hash_map_real_set;
	((GeeAbstractMapClass *) klass)->unset = gee_hash_map_real_unset;
	((GeeAbstractMapClass *) klass)->clear = gee_hash_map_real_clear;
	((GeeAbstractMapClass *) klass)->map_iterator = gee_hash_map_real_map_iterator;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_size = gee_hash_map_real_get_size;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_read_only = gee_hash_map_real_get_read_only;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_keys = gee_hash_map_real_get_keys;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_values = gee_hash_map_real_get_values;
	GEE_ABSTRACT_MAP_CLASS (klass)->get_entries = gee_hash_map_real_get_entries;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_hash_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_hash_map_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEYS, g_param_spec_object ("keys", "keys", "keys", GEE_TYPE_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUES, g_param_spec_object ("values", "values", "values", GEE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_ENTRIES, g_param_spec_object ("entries", "entries", "entries", GEE_TYPE_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_hash_map_instance_init (GeeHashMap * self) {
	self->priv = GEE_HASH_MAP_GET_PRIVATE (self);
	self->priv->_stamp = 0;
}


static void gee_hash_map_finalize (GObject* obj) {
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_HASH_MAP, GeeHashMap);
	gee_abstract_map_clear ((GeeAbstractMap*) self);
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
	_gee_functions_hash_data_func_closure_unref0 (self->priv->_key_hash_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_key_equal_func);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_value_equal_func);
	G_OBJECT_CLASS (gee_hash_map_parent_class)->finalize (obj);
}


/**
 * Hash table implementation of the {@link Map} interface.
 *
 * This implementation is better fit for highly heterogenous key values.
 * In case of high key hashes redundancy or higher amount of data prefer using
 * tree implementation like {@link TreeMap}.
 *
 * @see TreeMap
 */
GType gee_hash_map_get_type (void) {
	static volatile gsize gee_hash_map_type_id__volatile = 0;
	if (g_once_init_enter (&gee_hash_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMap), 0, (GInstanceInitFunc) gee_hash_map_instance_init, NULL };
		GType gee_hash_map_type_id;
		gee_hash_map_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_MAP, "GeeHashMap", &g_define_type_info, 0);
		g_once_init_leave (&gee_hash_map_type_id__volatile, gee_hash_map_type_id);
	}
	return gee_hash_map_type_id__volatile;
}


static void _vala_gee_hash_map_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_HASH_MAP, GeeHashMap);
	switch (property_id) {
		case GEE_HASH_MAP_SIZE:
		g_value_set_int (value, gee_abstract_map_get_size ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_map_get_read_only ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_KEYS:
		g_value_take_object (value, gee_abstract_map_get_keys ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_VALUES:
		g_value_take_object (value, gee_abstract_map_get_values ((GeeAbstractMap*) self));
		break;
		case GEE_HASH_MAP_ENTRIES:
		g_value_take_object (value, gee_abstract_map_get_entries ((GeeAbstractMap*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_hash_map_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_HASH_MAP, GeeHashMap);
	switch (property_id) {
		case GEE_HASH_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



